@@ -1,6 +1,5 @@
 /**************************************************
-opengate Mac addr auth program
-
+OpengateM - a MAC address authentication system
    module for local work database
 
 Copyright (C) 2011 Opengate Project Team
@@ -25,43 +24,80 @@ Email: watanaby@is.saga-u.ac.jp
 #include "opengatemd.h"
 #include <sqlite3.h>
 
+static int sqliteBusyTimeout=100;  /* value used in sqite3_busy_timeout() */
+static sqlite3 *dbMd; /* handle for opengatemd.db */
+
+/*******************************************************************
+ read sqlite busy timeout value from conf and set to static variable
+*******************************************************************/
+int setupSqliteBusyTimeoutValue(void){
+
+  char *str;
+  
+  /* if set in conf, use the value. if not, use the above default. */
+  str=GetConfValue("SqliteBusyTimeout");
+  if(str!=NULL) sqliteBusyTimeout=atoi(str);
+
+  return sqliteBusyTimeout;
+}
+
 /********************************************
 initialize work db implemented with sqlite
 ********************************************/
 int initWorkDb(void){
-  sqlite3 *db;
+
   char *pErrMsg=NULL;
 
   /* SQL CREATE COMMAND */
   char *createCmd1="CREATE TABLE IF NOT EXISTS sessionmd "
-    "(ipAddress TEXT PRIMARY KEY, "
+    "(macAddress TEXT PRIMARY KEY, "
     "userId TEXT, extraId TEXT, openTime INTEGER, checkTime INTEGER, "
-    "macAddress TEXT, ruleNumber INTEGER)";
+    "ruleNumber INTEGER)";
   char *createCmd2="CREATE TABLE IF NOT EXISTS macinfo "
     "(macAddress TEXT PRIMARY KEY ON CONFLICT REPLACE, "
     "detectTime INTEGER, ttl INTEGER, isNat INTEGER)";
+  char *createCmd3="CREATE TABLE IF NOT EXISTS macippair "
+    "(macAddress TEXT, "
+    "ipAddress TEXT, findTime INTEGER)";
+
+  /* setup static variable value for SqLite3_busy_timeout from conf */
+  SetupSqliteBusyTimeoutValue();
 
   /* Open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
+  if(sqlite3_open(GetConfValue("SqliteDbMd"),&dbMd)!=SQLITE_OK){
     err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(dbMd, sqliteBusyTimeout);
 
   /* create table1 */
-  if(sqlite3_exec(db, createCmd1, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, createCmd1, NULL, NULL, &pErrMsg)!=SQLITE_OK){
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
     terminateProg(0);
   }
 
   /* create table2 */
-  if(sqlite3_exec(db, createCmd2, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, createCmd2, NULL, NULL, &pErrMsg)!=SQLITE_OK){
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
     terminateProg(0);
   }
 
-  /* Sqlite close */
-  sqlite3_close(db);
+  /* create table3 */
+  if(sqlite3_exec(dbMd, createCmd3, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+    err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
+    terminateProg(0);
+  }
+
+  return TRUE;
+}
+
+/********************************************
+finalize work db implemented with sqlite
+********************************************/
+int finalizeWorkDb(void){
+
+  /* Close db for opengatemd */
+  sqlite3_close(dbMd);
 
   return TRUE;
 }
@@ -69,33 +105,25 @@ int initWorkDb(void){
 /************************************************************
  insert session info to db at start     
 ************************************************************/
-int insertSessionToWorkDb(char* ipAddress, char* userId, char* extraId, 
-			char* macAddress, int ruleNumber){
+int insertSessionToWorkDb(char* macAddress, char* userId, char* extraId, 
+			int ruleNumber){
 
   int rc;
-  sqlite3 *db;
   char *pErrMsg=NULL;
 
   /* SQL INSERT COMMAND, where %x is replaced in snprintf */
   char *insertFormat="INSERT INTO sessionmd "
-    "(ipAddress, userId, extraId, openTime, checkTime, macAddress, ruleNumber) "
-    "values ('%s','%s','%s', %d, %d,'%s', %d)";
+    "(macAddress, userId, extraId, openTime, checkTime, ruleNumber) "
+    "values ('%s','%s','%s', %d, %d, %d)";
   char *insertCmd;
   int resultFlag=TRUE;
 
-  /* Open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-
   /* Prepare insert command */
-  insertCmd=sqlite3_mprintf(insertFormat, ipAddress, userId,extraId, 
-			    time(NULL), time(NULL), macAddress, ruleNumber);
+  insertCmd=sqlite3_mprintf(insertFormat, macAddress, userId,extraId, 
+			    time(NULL), time(NULL), ruleNumber);
 
   /* Execute insert to sqlite */
-  if((rc=sqlite3_exec(db, insertCmd, NULL, NULL, &pErrMsg))!=SQLITE_OK){
+  if((rc=sqlite3_exec(dbMd, insertCmd, NULL, NULL, &pErrMsg))!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
   }
@@ -103,38 +131,27 @@ int insertSessionToWorkDb(char* ipAddress, char* userId, char* extraId,
   /*Memory free for sqlite3 string */
   sqlite3_free(insertCmd);
 
-  /* Sqlite close */
-  sqlite3_close(db);
-
   return resultFlag;
 }
 
 /*************************************************************
  update checkTime to time now
 *************************************************************/
-int updateCheckTimeInWorkDb(char* ipAddress){
+int updateCheckTimeInWorkDb(char* macAddress){
 
-  sqlite3 *db;
   char *pErrMsg=NULL;
 
   /* SQL UPDATE COMMAND, where %x is replaced in mprintf */
   char *updateFormat="UPDATE sessionmd "
-    "SET checkTime=%d WHERE ipAddress='%s'";
+    "SET checkTime=%d WHERE macAddress='%s'";
   char *updateCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-  
   /* prepare command */
-  updateCmd=sqlite3_mprintf(updateFormat, time(NULL), ipAddress);
+  updateCmd=sqlite3_mprintf(updateFormat, time(NULL), macAddress);
 
   /* execute update to sqlite */
-  if(sqlite3_exec(db, updateCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, updateCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
   }
@@ -142,37 +159,26 @@ int updateCheckTimeInWorkDb(char* ipAddress){
   /*memory free for sqlite3 string */
   sqlite3_free(updateCmd);
 
-  /* sqlite close */
-  sqlite3_close(db);
-
   return resultFlag;
 }
 
 /*************************************************************
- delete session info to db at stop     
+ delete session info in db at stop     
 *************************************************************/
-int delSessionFromWorkDb(char* ipAddress){
+int delSessionFromWorkDb(char* macAddress){
 
-  sqlite3 *db;
   char *pErrMsg=NULL;
 
   /* SQL DELETE COMMAND, where %x is replaced in mprintf */
-  char *deleteFormat="DELETE FROM sessionmd WHERE ipAddress='%s'";
+  char *deleteFormat="DELETE FROM sessionmd WHERE macAddress='%s'";
   char *deleteCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-
   /* prepare command */
-  deleteCmd=sqlite3_mprintf(deleteFormat, ipAddress);
+  deleteCmd=sqlite3_mprintf(deleteFormat, macAddress);
 
   /* execute delete */
-  if(sqlite3_exec(db, deleteCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, deleteCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
   }
@@ -180,65 +186,53 @@ int delSessionFromWorkDb(char* ipAddress){
   /*memory free for sqlite3 string */
   sqlite3_free(deleteCmd);
 
-  /* sqlite close */
-  sqlite3_close(db);
-  return TRUE;
+  return resultFlag;
 }
 
 /************************************************
- get info for active session about ip addr from db  
-input = ipAddress, output = others
+ get info for active session about mac addr from db  
+input = macAddress, output = others
 *************************************************/
-int getSessionFromWorkDb(char* ipAddress, char* userId, char* extraId, 
-			 int* openTime, int* checkTime, char *macAddress,
+int getSessionFromWorkDb(char* macAddress, char* userId, char* extraId, 
+			 int* openTime, int* checkTime, 
 			 int* ruleNumber){
 
-  sqlite3 *db;
   sqlite3_stmt *stmt;
  
   /* SQL SELECT COMMAND, where %x is replaced in snprintf */
   char *selectFormat="SELECT userId, extraId, openTime, checkTime, "
-    "macAddress, ruleNumber FROM sessionmd WHERE ipAddress='%s'";
+    "ruleNumber FROM sessionmd WHERE macAddress='%s'";
   char *selectCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    return FALSE;
-  }
-
   /* prepare command string */
-  selectCmd=sqlite3_mprintf(selectFormat, ipAddress);
+  selectCmd=sqlite3_mprintf(selectFormat, macAddress);
   
   /* compile to internal statement */
-  if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
+  if(sqlite3_prepare(dbMd, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
 
     /* finalize */
     sqlite3_free(selectCmd);
     sqlite3_finalize(stmt);
-    sqlite3_close(db);
+
     return FALSE;
   }
 
   /* get first record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(userId, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
-    strncpy(extraId, (char*)sqlite3_column_text(stmt, 1), USERMAXLN);
+    strlcpy(userId, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
+    strlcpy(extraId, (char*)sqlite3_column_text(stmt, 1), USERMAXLN);
     *openTime=(int)sqlite3_column_int(stmt, 2);
     *checkTime=(int)sqlite3_column_int(stmt, 3);
-    strncpy(macAddress, (char*)sqlite3_column_text(stmt, 4), ADDRMAXLN);
-    *ruleNumber=(int)sqlite3_column_int(stmt, 5);
+    *ruleNumber=(int)sqlite3_column_int(stmt, 4);
     resultFlag=TRUE;
   }else{
     userId[0]='\0';
     extraId[0]='\0';
     *openTime=0;
     *checkTime=0;
-    macAddress[0]='\0';
     *ruleNumber=0;
     resultFlag=FALSE;
   }
@@ -246,7 +240,6 @@ int getSessionFromWorkDb(char* ipAddress, char* userId, char* extraId,
   /* finalize */
   sqlite3_free(selectCmd);
   sqlite3_finalize(stmt);
-  sqlite3_close(db);
   
   return resultFlag;
 }
@@ -260,7 +253,6 @@ if delayed=FALSE, close all sessions without delay
 *************************************************/
 int delUselessSessionsInWorkDb(int delayed){
 
-  sqlite3 *db;
   char *pErrMsg=NULL;
   int uselessLimitTime;
 
@@ -271,25 +263,18 @@ int delUselessSessionsInWorkDb(int delayed){
   if(!delayed) uselessLimitTime = time(NULL);
 
   /* SQL SELECT COMMAND, where %x is replaced in snprintf */
-  char *selectFormat="SELECT ruleNumber, userId, extraId, ipAddress, "
+  char *selectFormat="SELECT ruleNumber, userId, extraId, "
     "macAddress, openTime FROM sessionmd WHERE checkTime<%d";
   char *deleteFormat="DELETE FROM sessionmd WHERE checkTime<%d";
   char *selectCmd;
   char *deleteCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-
   /* prepare command string for select */
   selectCmd=sqlite3_mprintf(selectFormat, uselessLimitTime);
 
   /* exec command, callback function = CloseSession */
-  if(sqlite3_exec(db, selectCmd, CloseSession, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, selectCmd, CloseSession, NULL, &pErrMsg)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite_exec:%s",__FILE__,__LINE__, pErrMsg);
   }
@@ -298,7 +283,7 @@ int delUselessSessionsInWorkDb(int delayed){
   deleteCmd=sqlite3_mprintf(deleteFormat, uselessLimitTime);
 
   /* exec command  */
-  if(sqlite3_exec(db, deleteCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+  if(sqlite3_exec(dbMd, deleteCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite_exec:%s",__FILE__,__LINE__, pErrMsg);
   }
@@ -306,55 +291,45 @@ int delUselessSessionsInWorkDb(int delayed){
   /* finalize */
   sqlite3_free(selectCmd);
   sqlite3_free(deleteCmd);
-  sqlite3_close(db);
 
   return resultFlag;
 }
 
 /************************************************
-get list of sessions from DB (create hash table)
+get list of sessions from DB and create hash table
 for opengatemd 
- key=ipAddress, value=0 
+ key=macAddress, value=0 
 *************************************************/
 int getSessionTableFromWorkDb(DB* sessionTable){
   DBT hashKey;
   DBT hashVal;
-  sqlite3 *db=NULL;
   sqlite3_stmt *stmt=NULL;
   int resultFlag=FALSE;
-  char ipAddress[ADDRMAXLN];
+  char macAddress[ADDRMAXLN];
   int zero=0;
  
-  /* SQL SELECT COMMAND */
-  char *selectCmd="SELECT ipAddress FROM sessionmd";
-
-  /* open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
+  /* SQL SELECT COMMAND to get all mac address in session table */
+  char *selectCmd="SELECT macAddress FROM sessionmd";
 
   /* compile to internal statement */
-  if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
+  if(sqlite3_prepare(dbMd, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
     err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
 
     /* finalize */
     sqlite3_finalize(stmt);
-    sqlite3_close(db);
     return FALSE;
   }
 
   /* get result rows */
   while(sqlite3_step(stmt)==SQLITE_ROW){
     resultFlag=TRUE;
-    strncpy(ipAddress,(char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
+    strlcpy(macAddress,(char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
 
     /* put to hash table */
     hashVal.data = &zero;
     hashVal.size = sizeof(int);    
-    hashKey.data = ipAddress;
-    hashKey.size = strlen(ipAddress)+1;
+    hashKey.data = macAddress;
+    hashKey.size = strlen(macAddress)+1;
     if(sessionTable->put(sessionTable, &hashKey, &hashVal, 0) == -1) {
       err_msg("ERR at %s#%d: fail to put into hash table",__FILE__,__LINE__);
     }
@@ -362,94 +337,18 @@ int getSessionTableFromWorkDb(DB* sessionTable){
 
   /* finalize */
   sqlite3_finalize(stmt);
-  sqlite3_close(db);
-
-  return resultFlag;
-}
-
-/************************************************
-get list of sessions from DB (create hash table)
-for opengate (NOT OpengateMd)
- key=ipAddress, value=pid(processID)
-*************************************************/
-int getSessionTableFromOpengateWorkDb(DB* sessionTable){
-  DBT hashKey;
-  DBT hashVal;
-  sqlite3 *db=NULL;
-  sqlite3_stmt *stmt=NULL;
-  int resultFlag=FALSE;
-  char ipAddress4[ADDRMAXLN];
-  char ipAddress6[ADDRMAXLN];
-  int pid;
- 
-  /* SQL SELECT COMMAND */
-  char *selectCmd="SELECT clientAddr4, clientAddr6, pid "
-    "FROM session where closeTime='-'";
-
-  /* open sqlite for opengate (if cannot open, ignore) */
-  if(sqlite3_open(GetConfValue("SqliteDb"),&db)!=SQLITE_OK){
-    sqlite3_close(db);
-    return FALSE;
-  }
-
-  /* compile to internal statement */
-  if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
-
-    /* finalize */
-    sqlite3_finalize(stmt);
-    sqlite3_close(db);
-    return FALSE;
-  }
-
-  /* get result rows */
-  while(sqlite3_step(stmt)==SQLITE_ROW){
-    resultFlag=TRUE;
-    strncpy(ipAddress4,(char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
-    strncpy(ipAddress6,(char*)sqlite3_column_text(stmt, 1), ADDRMAXLN);
-    pid=(int)sqlite3_column_int(stmt, 2);
-
-    /* put to hash table */
-    /* if set ipv4 addr, enter it to table */
-    if(!isNull(ipAddress4)){
-      hashVal.data = &pid;
-      hashVal.size = sizeof(int);    
-      hashKey.data = ipAddress4;
-      hashKey.size = strlen(ipAddress4)+1;
-      if(sessionTable->put(sessionTable, &hashKey, &hashVal, 0) == -1) {
-	err_msg("ERR at %s#%d: fail to put into hash table",__FILE__,__LINE__);
-      }
-    }
-
-    /* if set ipv6 addr, enter it to table */
-    /* in db, this field may contain ipv4 addr */
-    if(!isNull(ipAddress6) && strchr(ipAddress6,':')!=NULL){
-      hashVal.data = &pid;
-      hashVal.size = sizeof(int);    
-      hashKey.data = ipAddress6;
-      hashKey.size = strlen(ipAddress6)+1;
-      if(sessionTable->put(sessionTable, &hashKey, &hashVal, 0) == -1) {
-	err_msg("ERR at %s#%d: fail to put into hash table",__FILE__,__LINE__);
-      }
-    }
-  }
-
-  /* finalize */
-  sqlite3_finalize(stmt);
-  sqlite3_close(db);
 
   return resultFlag;
 }
 
 
 /**********************************
- put out detected mac info to db 
+ put out detected mac info to macinfo table in work db
 for checking nat 
 **********************************/
 int putMacInfoToWorkDb(char* macAddress, int ttl, int isNat){
 
   int rc;
-  sqlite3 *db;
   char *pErrMsg=NULL;
 
   /* SQL INSERT COMMAND, where %x is replaced in snprintf */
@@ -459,18 +358,11 @@ int putMacInfoToWorkDb(char* macAddress, int ttl, int isNat){
   char *insertCmd;
   int resultFlag=TRUE;
 
-  /* Open sqlite */
-  if(sqlite3_open(GetConfValue("SqliteDbMd"),&db)!=SQLITE_OK){
-    err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-
   /* Prepare insert command */
   insertCmd=sqlite3_mprintf(insertFormat, macAddress, time(NULL), ttl, isNat);
 
   /* Execute insert to sqlite */
-  if((rc=sqlite3_exec(db, insertCmd, NULL, NULL, &pErrMsg))!=SQLITE_OK){
+  if((rc=sqlite3_exec(dbMd, insertCmd, NULL, NULL, &pErrMsg))!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
   }
@@ -478,44 +370,81 @@ int putMacInfoToWorkDb(char* macAddress, int ttl, int isNat){
   /*Memory free for sqlite3 string */
   sqlite3_free(insertCmd);
 
-  /* Sqlite close */
-  sqlite3_close(db);
-
   return resultFlag;
 }
 
-/************************************
-is the rule number active in opengate session table
-************************************/
-int isActiveRuleInOpengateWorkDb(int ruleNumber){
 
-  sqlite3 *db;
+/**********************************
+ get mac info from macinfo table in work db 
+**********************************/
+int getMacInfoFromWorkDb(char* macAddress, char* detectTimeStr, int* pTtl){
+
   sqlite3_stmt *stmt;
- 
+
   /* SQL SELECT COMMAND, where %x is replaced in snprintf */
-  char *selectFormat="SELECT * FROM session "
-    " WHERE (ruleNumber6=%d or ruleNumber4=%d) and closeTime='-'";
+  char *selectFormat="SELECT datetime(detectTime,'unixepoch','localtime'), "
+    " ttl FROM macinfo WHERE macAddress='%s' ";
   char *selectCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite for opengate (if cannot open, ignore) */
-  if(sqlite3_open(GetConfValue("SqliteDb"),&db)!=SQLITE_OK){
-    sqlite3_close(db);
-    return FALSE;
-  }
+  /* set default value */
+  *pTtl=0;
+  detectTimeStr[0]='?';
+  detectTimeStr[1]='\0';
+
+  /* Prepare select command */
+  selectCmd=sqlite3_mprintf(selectFormat, macAddress);
 
-  /* prepare command string */
-  selectCmd=sqlite3_mprintf(selectFormat, ruleNumber, ruleNumber);
-  
   /* compile to internal statement */
-  if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
+  if(sqlite3_prepare(dbMd, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
+    resultFlag=FALSE;
+    err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
+
+    /* finalize */
+    sqlite3_free(selectCmd);
+    sqlite3_finalize(stmt);
+    return FALSE;
+  }
+
+  /* get first record */
+  if(sqlite3_step(stmt)==SQLITE_ROW){
+    strlcpy(detectTimeStr, (char*)sqlite3_column_text(stmt, 0), WORDMAXLN);
+    *pTtl=(int)sqlite3_column_int(stmt, 1);
+    resultFlag=TRUE;
+  }else{
+    resultFlag=FALSE;
+  }
+
+  /* finalize */
+  sqlite3_free(selectCmd);
+  sqlite3_finalize(stmt);
+  return resultFlag;
+}
+
+/************************************
+is the rule number active in opengatemd session table
+************************************/
+int isActiveRuleInWorkDb(int ruleNumber){
+
+  sqlite3_stmt *stmt;
+ 
+  /* SQL SELECT COMMAND, where %x is replaced in snprintf */
+  char *selectFormat="SELECT * FROM sessionmd "
+    " WHERE ruleNumber=%d";
+  char *selectCmd;
+  int resultFlag=TRUE;
+
+  /* prepare command string */
+  selectCmd=sqlite3_mprintf(selectFormat, ruleNumber);
+  
+  /* compile to internal statement */
+  if(sqlite3_prepare(dbMd, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
 
     /* finalize */
     sqlite3_free(selectCmd);
     sqlite3_finalize(stmt);
-    sqlite3_close(db);
     return FALSE;
   }
 
@@ -526,121 +455,113 @@ int isActiveRuleInOpengateWorkDb(int ruleNumber){
   /* finalize */
   sqlite3_free(selectCmd);
   sqlite3_finalize(stmt);
-  sqlite3_close(db);
-  
   return resultFlag;
 }
 
-
-/*************************************************************
- set close time to session table in opengate db
-*************************************************************/
-int setCloseTimeToOpengateWorkDb(char* ipAddress){
-
-  sqlite3 *db;
-  char *pErrMsg=NULL;
-
-  /* SQL UPDATE COMMAND, where %x is replaced in mprintf */
-   char *updateFormat="UPDATE session "
-     " SET closeTime=datetime(%d,'unixepoch','localtime') "
-     " WHERE closeTime='-' and (clientAddr4='%s' or clientAddr6='%s')";
-  char *updateCmd;
-  int resultFlag=TRUE;
-
-  /* open sqlite for opengate (if cannot open, ignore) */
-  if(sqlite3_open(GetConfValue("SqliteDb"),&db)!=SQLITE_OK){
-    sqlite3_close(db);
-    terminateProg(0);
-  }
-
-  /* prepare command */
-  updateCmd=sqlite3_mprintf(updateFormat, time(NULL), ipAddress, ipAddress);
-
-  /* execute update */
-  if(sqlite3_exec(db, updateCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
-    resultFlag=FALSE;
-    err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
-  }
-
-  /*memory free for sqlite3 string */
-  sqlite3_free(updateCmd);
-
-  /* sqlite close */
-  sqlite3_close(db);
-  return TRUE;
-}
-
-/************************************************
- get info for active session about ip addr from opengate db  
-input = ipAddress, output = others
-*************************************************/
-int getSessionFromOpengateWorkDb(char* ipAddress, char* userid, 
-			 char* openTime, char *macAddress, int* ruleNumber){
-
-  sqlite3 *db;
+/********************************************
+Is the MAC-IP pair found in work db
+********************************************/
+int isFoundMacIpPairInWorkDb(char* macAddress, char* ipAddress){
   sqlite3_stmt *stmt;
  
   /* SQL SELECT COMMAND, where %x is replaced in snprintf */
-  char *selectFormat4="SELECT userid, openTime, "
-    "macAddr, ruleNumber4 FROM session "
-    "WHERE clientAddr4='%s' and closeTime='-'";
-  char *selectFormat6="SELECT userid, openTime, "
-    "macAddr, ruleNumber6 FROM session "
-    " WHERE clientAddr6='%s' and closeTime='-'";
+  char *selectFormat="SELECT * FROM macippair "
+    " WHERE macAddress='%s' AND ipAddress='%s'";
   char *selectCmd;
   int resultFlag=TRUE;
 
-  /* open sqlite for opengate (if cannot open, ignore) */
-  if(sqlite3_open(GetConfValue("SqliteDb"),&db)!=SQLITE_OK){
-    sqlite3_close(db);
-    return FALSE;
-  }
-
-  /* prepare command string for IPv4 or IPv6 */
-  if(strchr(ipAddress, ':')==NULL){
-    selectCmd=sqlite3_mprintf(selectFormat4, ipAddress);
-  }else{
-    selectCmd=sqlite3_mprintf(selectFormat6, ipAddress);
-  }
+  /* prepare command string */
+  selectCmd=sqlite3_mprintf(selectFormat, macAddress, ipAddress);
   
   /* compile to internal statement */
-  if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
+  if(sqlite3_prepare(dbMd, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
     resultFlag=FALSE;
     err_msg("ERR at %s#%d: sqlite3_prepare",__FILE__,__LINE__);
 
     /* finalize */
     sqlite3_free(selectCmd);
     sqlite3_finalize(stmt);
-    sqlite3_close(db);
     return FALSE;
   }
 
   /* get first record */
-  if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(userid, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
-    strncpy(openTime, (char*)sqlite3_column_text(stmt, 1), USERMAXLN);
-    strncpy(macAddress, (char*)sqlite3_column_text(stmt, 2), ADDRMAXLN);
-    *ruleNumber=(int)sqlite3_column_int(stmt, 3);
-    resultFlag=TRUE;
-  }else{
-    userid[0]='\0';
-    *openTime=0;
-    macAddress[0]='\0';
-    *ruleNumber=0;
-    resultFlag=FALSE;
-  }
+  if(sqlite3_step(stmt)==SQLITE_ROW) resultFlag=TRUE;
+  else resultFlag=FALSE;
 
   /* finalize */
   sqlite3_free(selectCmd);
   sqlite3_finalize(stmt);
-  sqlite3_close(db);
-  
   return resultFlag;
 }
 
+
+/********************************************
+Insert MAC-IP pair to work db
+********************************************/
+int putMacIpPairToWorkDb(char* macAddress, char* ipAddress){
+  int rc;
+  char *pErrMsg=NULL;
+
+  /* SQL INSERT COMMAND, where %x is replaced in snprintf */
+  char *insertFormat="INSERT INTO macippair "
+    "(macAddress, ipAddress, findTime) "
+    "values ('%s','%s', %d)";
+  char *insertCmd;
+  int resultFlag=TRUE;
+
+  /* Prepare insert command */
+  insertCmd=sqlite3_mprintf(insertFormat, macAddress, ipAddress, time(NULL));
+
+  /* Execute insert to sqlite */
+  if((rc=sqlite3_exec(dbMd, insertCmd, NULL, NULL, &pErrMsg))!=SQLITE_OK){
+    resultFlag=FALSE;
+    err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
+  }
+
+  /*Memory free for sqlite3 string */
+  sqlite3_free(insertCmd);
+
+  return resultFlag;
+}
+
+/********************************************
+Delete the mac-ip pairs in work db for the mac
+********************************************/
+int delMacIpPairsInWorkDb(char* macAddress){
+  char *pErrMsg=NULL;
+
+  /* SQL DELETE COMMAND, where %x is replaced in mprintf */
+  char *deleteFormat="DELETE FROM macippair WHERE macAddress='%s'";
+  char *deleteCmd;
+  int resultFlag=TRUE;
+
+  /* prepare command */
+  deleteCmd=sqlite3_mprintf(deleteFormat, macAddress);
+
+  /* execute delete */
+  if(sqlite3_exec(dbMd, deleteCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
+    resultFlag=FALSE;
+    err_msg("ERR at %s#%d: sqlite3_exec: %s",__FILE__,__LINE__,pErrMsg);
+  }
+
+  /*memory free for sqlite3 string */
+  sqlite3_free(deleteCmd);
+
+  return resultFlag;
+
+}
+
 /*********************************************************
  routines for debugging output
 *********************************************************/
+int SetupSqliteBusyTimeoutValue(void){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>setupSqliteBusyTimeoutValue()");
+  ret=setupSqliteBusyTimeoutValue();
+  if(debug>1) err_msg("DEBUG:(%d)<=setupSqliteBusyTimeoutValue()",ret);
+  return ret;
+}
+
 int InitWorkDb(void){
   int ret;
   if(debug>1) err_msg("DEBUG:=>initWorkDb( )");
@@ -648,41 +569,49 @@ int InitWorkDb(void){
   if(debug>1) err_msg("DEBUG:(%d)<=initWorkDb( )",ret);
   return ret;
 }
-int InsertSessionToWorkDb(char* ipAddress, char* userId, char* extraId, 
-			char* macAddress, int ruleNumber){
+
+int FinalizeWorkDb(void){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>insertSessionToWorkDb(%s,%s,%s,%s,%d)",
-		      ipAddress, userId, extraId, macAddress, ruleNumber);
-  ret = insertSessionToWorkDb(ipAddress, userId, extraId, macAddress, ruleNumber);
+  if(debug>1) err_msg("DEBUG:=>finalizeWorkDb( )");
+  ret = finalizeWorkDb();
+  if(debug>1) err_msg("DEBUG:(%d)<=finalizeWorkDb( )",ret);
+  return ret;
+}
+
+int InsertSessionToWorkDb(char* macAddress, char* userId, char* extraId, 
+			 int ruleNumber){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>insertSessionToWorkDb(%s,%s,%s,%d)",
+		      macAddress, userId, extraId, ruleNumber);
+  ret = insertSessionToWorkDb(macAddress, userId, extraId, ruleNumber);
   if(debug>1) err_msg("DEBUG:(%d)<=insertSessionToWorkDb( )",ret);
   return ret;
 }
 
-int UpdateCheckTimeInWorkDb(char* ipAddress){
+int UpdateCheckTimeInWorkDb(char* macAddress){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>updateCheckTimeInWorkDb(%s)", ipAddress);
-  ret = updateCheckTimeInWorkDb(ipAddress);
+  if(debug>1) err_msg("DEBUG:=>updateCheckTimeInWorkDb(%s)", macAddress);
+  ret = updateCheckTimeInWorkDb(macAddress);
   if(debug>1) err_msg("DEBUG:(%d)<=updateCheckTimeInWorkDb( )",ret);
   return ret;
 }
 
-int DelSessionFromWorkDb(char* ipAddress){
+int DelSessionFromWorkDb(char* macAddress){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>delSessionFromWorkDb(%s)", ipAddress);
-  ret = delSessionFromWorkDb(ipAddress);
+  if(debug>1) err_msg("DEBUG:=>delSessionFromWorkDb(%s)", macAddress);
+  ret = delSessionFromWorkDb(macAddress);
   if(debug>1) err_msg("DEBUG:(%d)<=delSessionFromWorkDb( )",ret);
   return ret;
 }
-int GetSessionFromWorkDb(char* ipAddress, char* userId, char* extraId, 
-			 int* openTime, int* checkTime, char *macAddress,
-			 int* ruleNumber){
+
+int GetSessionFromWorkDb(char* macAddress, char* userId, char* extraId, 
+			 int* openTime, int* checkTime, int* ruleNumber){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>getSessionFromWorkDb(%s)", ipAddress);
-  ret = getSessionFromWorkDb(ipAddress, userId, extraId, openTime, 
-			       checkTime, macAddress, ruleNumber);
-  if(debug>1) err_msg("DEBUG:(%d)<=getSessionFromWorkDb(,%s,%s,%d,%d,%s,%d)",
-		      ret,userId,extraId,*openTime,*checkTime,macAddress,
-		      *ruleNumber);
+  if(debug>1) err_msg("DEBUG:=>getSessionFromWorkDb(%s)", macAddress);
+  ret = getSessionFromWorkDb(macAddress, userId, extraId, openTime, 
+			       checkTime, ruleNumber);
+  if(debug>1) err_msg("DEBUG:(%d)<=getSessionFromWorkDb(,%s,%s,%d,%d,%d)",
+		      ret,userId,extraId,*openTime,*checkTime, *ruleNumber);
   return ret;
 }
 
@@ -702,14 +631,6 @@ int GetSessionTableFromWorkDb(DB* sessionTable){
   return ret;
 }
 
-int GetSessionTableFromOpengateWorkDb(DB* sessionTable){
-  int ret;
-  if(debug>1) err_msg("DEBUG:=>getSessionTableFromOpengateWorkDb( )");
-  ret=getSessionTableFromOpengateWorkDb(sessionTable);
-  if(debug>1) err_msg("DEBUG:(%d)<=getSessionTableFromOpengateWorkDb( )", ret);
-  return ret;
-}
-
 int PutMacInfoToWorkDb(char* macAddress, int ttl, int isNat){
   int ret;
   if(debug>1) err_msg("DEBUG:=>putMacInfoToWorkDb(%s,%d,%d)",macAddress,ttl,isNat);
@@ -718,32 +639,47 @@ int PutMacInfoToWorkDb(char* macAddress, int ttl, int isNat){
   return ret;
 }
 
-int IsActiveRuleInOpengateWorkDb(int ruleNumber){
+int IsActiveRuleInWorkDb(int ruleNumber){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>isActiveRuleInOpengateWorkDb(%d)",ruleNumber);
-  ret=isActiveRuleInOpengateWorkDb(ruleNumber);
-  if(debug>1) err_msg("DEBUG:(%d)<=isActiveRuleInOpengateWorkDb( )", ret);
+  if(debug>1) err_msg("DEBUG:=>isActiveRuleInWorkDb(%d)",ruleNumber);
+  ret=isActiveRuleInWorkDb(ruleNumber);
+  if(debug>1) err_msg("DEBUG:(%d)<=isActiveRuleInWorkDb( )", ret);
   return ret;
 }
 
-int SetCloseTimeToOpengateWorkDb(char* ipAddress){
+int GetMacInfoFromWorkDb(char* macAddress, char* detectTimeStr, int* pTtl){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>setCloseTimeToOpengateWorkDb(%s)", ipAddress);
-  ret=setCloseTimeToOpengateWorkDb(ipAddress);
-  if(debug>1) err_msg("DEBUG:(%d)<=setCloseTimeToOpengateWorkDb( )", ret);
+  if(debug>1) err_msg("DEBUG:=>getMacInfoFromWorkDb(%s)", macAddress);
+  ret = getMacInfoFromWorkDb(macAddress, detectTimeStr, pTtl);
+  if(debug>1) err_msg("DEBUG:(%d)<=getMacInfoFromWorkDb(,%s,%d)",
+		      ret, detectTimeStr, *pTtl);
   return ret;
 }
 
-int GetSessionFromOpengateWorkDb(char* ipAddress, char* userid, 
-			 char* openTime, char *macAddress, int* ruleNumber){
-
+int IsFoundMacIpPairInWorkDb(char* macAddress, char* ipAddress){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>getSessionFromOpengateWorkDb(%s)", ipAddress);
-  ret = getSessionFromOpengateWorkDb(ipAddress, userid, openTime, 
-			       macAddress, ruleNumber);
-  if(debug>1) err_msg("DEBUG:(%d)<=getSessionFromOpengateWorkDb(,%s,%s,%s,%d)",
-		      ret,userid,openTime,macAddress,*ruleNumber);
+  if(debug>1) err_msg("DEBUG:=>isFoundMacIpPairInWorkDb(%s,%s)",
+		      macAddress,ipAddress);
+  ret=isFoundMacIpPairInWorkDb(macAddress, ipAddress);
+  if(debug>1) err_msg("DEBUG:(%d)<=isfoundMacIpPairInWorkDb( )", ret);
+  return ret;
+}
+
+int PutMacIpPairToWorkDb(char* macAddress, char* ipAddress){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>putMacIpPairToWorkDb(%s,%s)",
+		      macAddress,ipAddress);
+  ret=putMacIpPairToWorkDb(macAddress, ipAddress);
+  if(debug>1) err_msg("DEBUG:(%d)<=putMacIpPairtoWorkDb( )", ret);
   return ret;
 }
 
 
+int DelMacIpPairsInWorkDb(char* macAddress){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>delMacIpPairsInWorkDb(%s)",
+		      macAddress);
+  ret=delMacIpPairsInWorkDb(macAddress);
+  if(debug>1) err_msg("DEBUG:(%d)<=delMacIpPairsInWorkDb( )", ret);
+  return ret;
+}
