@@ -1,5 +1,5 @@
 /**************************************************
-opengate mac addr auth program
+OpengateM - MAC address authentication system 
  module for Authentication of User
 
 Copyright (C) 1999 Opengate Project Team
@@ -43,18 +43,20 @@ int AuthLdap(char *userid, char *passwd);
 
 /****************************************
 get userid from anywhere (cookie/env/postdata)
+if not get, send back auth page.
 language indicates the one for web description
 ****************************************/
-int getUserId(char* requestStr, char* userId, char* extraId, char* language, int userType, char* cgiName, char* mailDefault){
+int getUserId(char* requestStr, char* userId, char* extraId, char* language, int userType, char* cgiName, char* mailDefault, char* redirectedUrl){
   int authResult=DENY;
   char useridfull[USERMAXLN]; /* userid@extraid */
   char cookie[SIDMAXLN];
   char password[USERMAXLN];
+  char* docName;
 
   /* default mail address is null */
   mailDefault[0]='\0';
 
-  /* if userid is found in cookie, accept auth */
+  /***** try to get userid from db corresponding to http cookie *****/
   switch(userType){
   case NORMALUSER:
     if(GetHttpCookie(cookie, GetConfValue("AuthUserCookie"))){
@@ -75,7 +77,7 @@ int getUserId(char* requestStr, char* userId, char* extraId, char* language, int
     break;
   }
 
-  /* if not auth by cookie, check env-var(shibboleth/httpbasic) */ 
+  /***** try to get uid from environment variables(shibboleth/httpbasic) *****/
   if(authResult==DENY){
 
     /* search shibboleth / httpbasic auth setting in conf */
@@ -95,8 +97,11 @@ int getUserId(char* requestStr, char* userId, char* extraId, char* language, int
       /* if reached to this(=shibboleth/httpbasic), get userid from env var */
       if(GetUserIdFromEnv(useridfull)){
 
-	/* if the user is found in accept user list, accept him, else deny */
+	/* split user@extra to user and extra. then search conf extra set */
 	SplitId(useridfull, userId, extraId);
+	SetupConfExtra(userId, extraId);
+
+	/* if the user is found in accept user list, accept him, else deny */
 	if(IsUserIdFoundInAcceptUsersList(userId)){
 	  authResult=ACCEPT;
 	  MakeMailDefault(userId, extraId, mailDefault);
@@ -112,21 +117,27 @@ int getUserId(char* requestStr, char* userId, char* extraId, char* language, int
     }
   }
 
-  /* if not auth by cookie and env-var, get user id from request str */
+  /***** try to get userid from request string *****/
   if(authResult==DENY){
+
+    /* if not get, in request string, send back auth page */
     if(!GetUserIdFromPostData(requestStr, useridfull, password)){
+
+      /* split user@extra to user and extra, then search conf extra set */
+      SplitId(useridfull, userId, extraId);
+      SetupConfExtra(userId, extraId);
       
-      /* if not get user id, send auth request page, and end */
-      if(userType==ADMINUSER) SetMessage(RequestAdminAuth);
-      PutAuthRequestPageToClient(language, cgiName);
+      /* select document content */
+      if(userType==ADMINUSER) docName=GetConfValue("AuthAdminDoc");
+      else docName=GetConfValue("AuthDoc");
+
+      /* put page and exit */
+      PutAuthRequestPageToClient(language, cgiName, docName, redirectedUrl);
       return FALSE;
     }
-  }
 
-  /* if not yet auth, check auth */
-  if(authResult==DENY){
-
-    /* split user@extra to user and extra */
+    /* reaching this line means that userid is in request string */
+    /* split user@extra to user and extra. then search conf extra set */
     SplitId(useridfull, userId, extraId);
     SetupConfExtra(userId, extraId);
 
@@ -134,8 +145,8 @@ int getUserId(char* requestStr, char* userId, char* extraId, char* language, int
     ResetAuthServerPointer();
     while(SelectNextAuthServer()){
       
-      /* if check normal user and admin auth server, goto next server */
-      /* if check admin user and not admin auth server, goto next server */
+      /* if check normal user and admin auth server, skip the server */
+      /* if check admin user and not admin auth server, skip the server */
       if(userType==NORMALUSER 
 	 && strcmp(GetConfValue("AuthServer/UserType"), "admin")==0) continue;
       if(userType==ADMINUSER
@@ -144,20 +155,20 @@ int getUserId(char* requestStr, char* userId, char* extraId, char* language, int
       /* authenticate the user with auth server. if deny, goto next server */
       if((authResult=AuthenticateUser(userId, password))==DENY) continue;
 
-      /* if user does not exist in the user list, goto next auth server */
+      /* if userid is not found in the user list in conf, goto next server */
       if(!IsUserIdFoundInAcceptUsersList(userId)){
 	authResult=DENY;
 	continue;
       }
 
-      /* if accepted, set mail default(used for warning mail) end search */
+      /* if accepted, set mail default(used for warning mail) and exit loop */
       if(authResult==ACCEPT){
 	MakeMailDefault(userId, extraId, mailDefault);
 	break;
       }
     }
 
-    /* if fail, put error */
+    /* if all check is failed, put error */
     if(authResult==DENY){
       SetMessage(NoInfoInDb); 
       PutDenyToClient(language);
@@ -178,18 +189,19 @@ int isUserIdFoundInAcceptUsersList(char* userId){
   char userIdPattern[WORDMAXLN];
 
   /* get accept users list. if not exist, return true(accept all users) */
-  strncpy(usersList,GetConfValue("AuthServer/AcceptUsers"),BUFFMAXLN);
+  strlcpy(usersList,GetConfValue("AuthServer/AcceptUsers"),BUFFMAXLN);
   if(isNull(usersList)) return TRUE;
 
   /* if userid is found in the usersList, return true */
   /* example of usersList is [user1 user2 user3 user4] */
 
-  /* regular expression match to "(^| )userid( |$)" */
+  /* regular expression matched to "(^| )userid( |$)" */
   /* meaning is [(head or space) userid-string (space or tail)] */
-  strncpy(userIdPattern, "(^| )", WORDMAXLN);
-  strncat(userIdPattern, userId, WORDMAXLN);
-  strncat(userIdPattern, "( |$)", WORDMAXLN);
-  return RegExMatch(usersList, userIdPattern); 
+ /* last-arg 0 of RegEx means ignore-case */
+  strlcpy(userIdPattern, "(^| )", WORDMAXLN);
+  strlcat(userIdPattern, userId, WORDMAXLN);
+  strlcat(userIdPattern, "( |$)", WORDMAXLN);
+  return RegExMatch(usersList, userIdPattern, 0);
 }
 
 
@@ -265,6 +277,11 @@ int authenticateUser(char *userid, char *passwd)
   }else if(strcmp(proto, "accept")==0){
     /* accept all request */
     authResult=ACCEPT;
+    
+  }else if(strcmp(proto, "splash")==0){
+    /* accept all request */
+    authResult=ACCEPT;
+    
   }else{
     err_msg("ERR at %s#%d: Unknown protocol:%s",__FILE__,__LINE__,proto);
     authResult=DENY;
@@ -284,19 +301,19 @@ void splitId(char* useridfull, char* userId, char* extraId)
 
   char* markPnt=NULL;
 
-  strncpy(useridSave, useridfull, USERMAXLN);
+  strlcpy(useridSave, useridfull, USERMAXLN);
 
   /* separate extraId from userid */
   markPnt=strchr(useridSave, *GetConfValue("UserIdSeparator"));
   if(markPnt==NULL){  
     /* separator mark not found */
-    strncpy(extraId,"",USERMAXLN);
-    strncpy(userId,useridSave,USERMAXLN);
+    strlcpy(extraId,"",USERMAXLN);
+    strlcpy(userId,useridSave,USERMAXLN);
   }else{
     /* pick up extraId */
     *markPnt='\0';
-    strncpy(extraId,markPnt+1,USERMAXLN);
-    strncpy(userId,useridSave,USERMAXLN);
+    strlcpy(extraId,markPnt+1,USERMAXLN);
+    strlcpy(userId,useridSave,USERMAXLN);
   }
 } 
 
@@ -306,10 +323,10 @@ concatinate userid and extraid
 char* concatUserId(char* useridfull, char* userId, char* extraId){
 
   /* set full userid */
-  strncpy(useridfull, userId,USERMAXLN);
+  strlcpy(useridfull, userId,USERMAXLN);
   if(!isNull(extraId)){
-    strncat(useridfull, GetConfValue("UserIdSeparator"), USERMAXLN);
-    strncat(useridfull, extraId, USERMAXLN);
+    strlcat(useridfull, GetConfValue("UserIdSeparator"), USERMAXLN);
+    strlcat(useridfull, extraId, USERMAXLN);
   }
   return useridfull;
 }
@@ -530,39 +547,73 @@ made from conf value at MailAttribute or MailDomain
 **************************************/
 int makeMailDefault(char* userId, char* extraId, char* mailDefault){
 
-  char* pMail=NULL;
+  char* pMailAttribute=NULL;
+  char* pMailDomain=NULL;
+  char* pMailAddr=NULL;
 
   /* set default null */
-  strncpy(mailDefault, "", BUFFMAXLN);
+  mailDefault[0]='\0';
+
+  /* get conf values for mail domain */
+  pMailAttribute=GetConfValue("AuthServer/MailAttribute");
+  pMailDomain=GetConfValue("AuthServer/MailDomain");
+  if(isNull(pMailAttribute) && isNull(pMailDomain)){
+      err_msg("ERR at %s#%d: Can't get mail attibute/domain setting from conf file.",__FILE__,__LINE__);
+    return FALSE;
+  }
 
   /* if set mail attribute, mail from the env variable in attr */
-  pMail=GetConfValue("AuthServer/MailAttribute");
-  if(!isNull(pMail)){
-    pMail=getenvEx(pMail,TRUE,FALSE); /* var list is permitted */
-    if(!isNull(pMail)){
-      strncpy(mailDefault, pMail, BUFFMAXLN);
+  if(!isNull(pMailAttribute)){
+    pMailAddr=getenvEx(pMailAttribute,TRUE,FALSE); /* var list is permitted */
+    if(!isNull(pMailAddr)){
+      strlcpy(mailDefault, pMailAddr, BUFFMAXLN);
       return TRUE;
+    }else{
+      err_msg("ERR at %s#%d: Can't get mail attibute from environment variable.",__FILE__,__LINE__);
+      return FALSE;
     }
   }
 
   /* if set mail domain, mail from userid+@+maildomain */
-  pMail=GetConfValue("AuthServer/MailDomain");
-  if(!isNull(pMail)){
-    strncpy(mailDefault, userId, BUFFMAXLN);
-    strncat(mailDefault, "@", BUFFMAXLN);
-    strncat(mailDefault, pMail, BUFFMAXLN);
+  if(!isNull(pMailDomain)){
+    strlcpy(mailDefault, userId, BUFFMAXLN);
+    strlcat(mailDefault, "@", BUFFMAXLN);
+    strlcat(mailDefault, pMailDomain, BUFFMAXLN);
     return TRUE;
+  }else{
+      err_msg("ERR at %s#%d: Can't get mail domain string from conf file.",__FILE__,__LINE__);
+    return FALSE;
   }
 
+  err_msg("ERR at %s#%d: Can't set mail. Check AuthServer/MailAttribute or AuthServer/MailDomain in conf.",__FILE__,__LINE__);
   return FALSE;  
 }
 
+/**********************************
+is AuthServer setting for normal user 'splash'
+**********************************/
+int isSplashOnlyManagement(void){
+  int splash=FALSE;
+
+  /* search AuthServer setting having Protocol=splash and UserType!=admin */
+  ResetAuthServerPointer();
+  while(SelectNextAuthServer()){
+    if((strcmp(GetConfValue("AuthServer/Protocol"), "splash")==0)&&
+       (strcmp(GetConfValue("AuthServer/UserType"), "admin")!=0) ){
+      splash=TRUE;
+      break;
+    }
+  }
+  ResetAuthServerPointer();
+  return splash;
+}
+  
 /**********************************/
 /**********************************/
-int GetUserId(char* requestStr, char* userId, char* extraId, char* language, int userType, char* cgiName, char* mailDefault){
+int GetUserId(char* requestStr, char* userId, char* extraId, char* language, int userType, char* cgiName, char* mailDefault, char* redirectedUrl){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>getUserId(%s,%s,%d)",requestStr,language,userType);
-  ret=getUserId(requestStr, userId, extraId, language, userType, cgiName, mailDefault);
+  if(debug>1) err_msg("DEBUG:=>getUserId(..,%s,%d)",language,userType);
+  ret=getUserId(requestStr, userId, extraId, language, userType, cgiName, mailDefault, redirectedUrl);
   if(debug>1) err_msg("DEBUG:(%d)<=getUserId(%s,%s,%s)",ret,userId,extraId,mailDefault);
   return ret;
 }
@@ -646,3 +697,13 @@ int MakeMailDefault(char* userId, char* extraId, char* mailDefault){
 
   return ret;
 }
+
+int IsSplashOnlyManagement(void){
+  int ret;
+
+  if(debug>1) err_msg("DEBUG:=>isSplashOnlyManagement()");
+  ret=isSplashOnlyManagement();
+  if(debug>1) err_msg("DEBUG:(%d)<=isSplashOnlyManagement",ret);
+
+  return ret;
+}
