@@ -1,5 +1,5 @@
 /**************************************************
-Opengate mac addr auth program
+OpengateM - MAC address authentication system 
  module for Communication through CGI 
 
 Copyright (C) 2011 Opengate Project Team
@@ -74,20 +74,53 @@ int getLangFromQueryString(char* language){
   if(isNull(getenv("QUERY_STRING"))) return FALSE;
 
   /* get html access parameter string */
-  strncpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
+  strlcpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
 
   /* split language in string [....&lang=ja&.....] */
   ptr=queryStr;
   while(ptr!=NULL){
     split(ptr, name, value, next);
     if(strstr(name[0], "lang")!=NULL){
-      strncpy(language, value[0], WORDMAXLN);
+      if(!isNull(value[0])) strlcpy(language, value[0], WORDMAXLN);
       found=TRUE;
     }
     ptr=next[0];
   }
   return found;
+}
 
+/********************************************/
+/* get redirected url form query string     */
+/********************************************/
+int getRedirectedUrlFromQueryString(char* redirectedUrl){
+
+  int found=FALSE;
+  char queryStr[BUFFMAXLN];
+  char *name[1];
+  char *value[1];
+  char *next[1];
+  char *ptr;
+
+  /* default is null */
+  redirectedUrl[0]='\0';
+
+  /* if no string, return default */
+  if(isNull(getenv("QUERY_STRING"))) return FALSE;
+
+  /* get html access parameter string */
+  strlcpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
+
+  /* split language in string [....&redirectedurl=xxxx&.....] */
+  ptr=queryStr;
+  while(ptr!=NULL){
+    split(ptr, name, value, next);
+    if(strstr(name[0], "redirectedurl")!=NULL){
+      if(!isNull(value[0])) strlcpy(redirectedUrl, value[0], BUFFMAXLN);
+      found=TRUE;
+    }
+    ptr=next[0];
+  }
+  return found;
 }
 
 /********************************************/
@@ -109,14 +142,14 @@ int getMacAddrFromQueryString(char* macAddress){
   if(isNull(getenv("QUERY_STRING"))) return FALSE;
 
   /* get html access parameter string */
-  strncpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
+  strlcpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
 
   /* split in string [....&macaddr=xxxxx&.....] */
   ptr=queryStr;
   while(ptr!=NULL){
     split(ptr, name, value, next);
     if(strstr(name[0], "macaddr")!=NULL){
-      strncpy(macAddress, value[0], WORDMAXLN);
+      strlcpy(macAddress, value[0], WORDMAXLN);
       found=TRUE;
     }
     ptr=next[0];
@@ -128,6 +161,7 @@ int getMacAddrFromQueryString(char* macAddress){
 
 /********************************************/
 /* get userid from environment variable     */
+/*  long userid: "userid@extraid" is set    */
 /********************************************/
 int getUserIdFromEnv(char *userid){
 
@@ -137,21 +171,33 @@ int getUserIdFromEnv(char *userid){
   /* if shibboleth or httpbasic, get uid from environment var */
   /* shibboleth */
   if(strcmp(GetConfValue("AuthServer/Protocol"), "shibboleth")==0){
+
+    /* get data from env-variables for uid and org attribute */
     pEnv=getenvEx(GetConfValue("AuthServer/UidAttribute"),TRUE,TRUE); 
     if(!isNull(pEnv)){
-      strncpy(userid, pEnv, USERMAXLN);
+      strlcpy(userid, pEnv, USERMAXLN);
 
-      /* if idp string can be get from env variable, concatinate it as uid@idp */
+      /* if orp string can be get from env var, concatenate it as uid@org */
       pEnv=getenvEx(GetConfValue("AuthServer/OrgAttribute"),TRUE,TRUE);
       if(!isNull(pEnv)){
-	strncat(userid, GetConfValue("UserIdSeparator"), USERMAXLN);
-	strncat(userid, pEnv, USERMAXLN);
+	strlcat(userid, GetConfValue("UserIdSeparator"), USERMAXLN);
+	strlcat(userid, pEnv, USERMAXLN);
       } 
       ret=TRUE;
-    }else{
+    }
+
+    /* get from env-variable for EPPN(edu person principal name) attribute */
+    else{
+      pEnv=getenvEx(GetConfValue("AuthServer/EppnAttribute"),TRUE,FALSE);
+      if(!isNull(pEnv)){
+	strlcat(userid, pEnv, USERMAXLN);
+	ret=TRUE;
+      }
+    }
+
+    if(ret==FALSE){
       err_msg("ERR at %s#%d: Cannot get user info from shibboleth",__FILE__,__LINE__);
-      ret=FALSE;
-      PutMessageToClient("Cannot get user info from shibboleth<br>Check shibboleth setting in .htaccess and other");
+      PutMessageToClient("Cannot get user info from shibboleth<br>Check shibboleth setting in .htaccess, conf and others");
       exit(0);
     }    
   }
@@ -159,7 +205,7 @@ int getUserIdFromEnv(char *userid){
   /* httpbasic */
   else if(strcmp(GetConfValue("AuthServer/Protocol"), "httpbasic")==0){
     if(!isNull(getenv("REMOTE_USER"))){
-      strncpy(userid,getenv("REMOTE_USER"),USERMAXLN);
+      strlcpy(userid,getenv("REMOTE_USER"),USERMAXLN);
       ret=TRUE;
     }else{
       err_msg("ERR at %s#%d: Cannot get user info from httpbasic",__FILE__,__LINE__);
@@ -187,7 +233,7 @@ int getUserIdFromPostData(char* requestStr, char* userid, char* password){
   if(isNull(requestStr)) return FALSE;
 
   /* copy it to work area */
-  strncpy(content, requestStr, BUFFMAXLN);
+  strlcpy(content, requestStr, BUFFMAXLN);
 
   /* split request item and execute the request */
   ptr=content;
@@ -198,18 +244,22 @@ int getUserIdFromPostData(char* requestStr, char* userid, char* password){
     
     /* copy to var */
     if(strcmp(name[0], "userid")==0){
-      strncpy(userid, value[0], USERMAXLN);
+      strlcpy(userid, value[0], USERMAXLN);
       decode(userid);
       ret=TRUE;
     }
     else if(strcmp(name[0], "password")==0){
-      strncpy(password, value[0], USERMAXLN);
+      strlcpy(password, value[0], USERMAXLN);
       decode(password);
     }
     
     /* shift pointer to next item */
     ptr=next[0];
   }
+
+  /* if password is found in request string, clear the string */
+  if(!isNull(password)) requestStr[0]='\0';
+
   return ret;
 }
 
@@ -242,7 +292,7 @@ int analyzeCheckRequest(char *content, int* status, char* macAddress)
 
     /* if item = macaddr */
     else if(strcmp(name[0], "macaddr")==0){
-      strncpy(macAddress, value[0], ADDRMAXLN);
+      strlcpy(macAddress, value[0], ADDRMAXLN);
       decode(macAddress);
     }
     
@@ -263,6 +313,9 @@ int analyzeRegisterRequest(char *content, char* macAddress, char* deviceName, ch
   char *next[1];
   char *ptr;
 
+  /* if null request, return */
+  if(isNull(content)) return FALSE;
+  
   /* set default */
   *macAddress='\0';
   *deviceName='\0';
@@ -277,25 +330,23 @@ int analyzeRegisterRequest(char *content, char* macAddress, char* deviceName, ch
 
     /* if item = macaddr */
     if(strcmp(name[0], "macaddr")==0){
-      strncpy(macAddress, value[0], ADDRMAXLN);
+      strlcpy(macAddress, value[0], ADDRMAXLN);
       decode(macAddress);
+
     }
     else if(strcmp(name[0], "device")==0){
-      strncpy(deviceName, value[0], WORDMAXLN);
+      strlcpy(deviceName, value[0], WORDMAXLN);
       decode(deviceName);
     }
     else if(strcmp(name[0], "mailaddr")==0){
-      strncpy(mailAddress, value[0], BUFFMAXLN);
+      strlcpy(mailAddress, value[0], BUFFMAXLN);
       decode(mailAddress);
     }
-     
+
     /* shift pointer to next item */
     ptr=next[0];
   }
 
-  /* if no data is inputed, return */
-  if(*deviceName=='\0' && *mailAddress=='\0') return FALSE;
-
   /* if illegal device name, return */
   if(*deviceName=='\0'){
     SetMessage(EmptyDeviceName); 
@@ -307,9 +358,11 @@ int analyzeRegisterRequest(char *content, char* macAddress, char* deviceName, ch
   }
 
   /* if illegal mail address, return */
-  if(!IsSafeString(mailAddress, BUFFMAXLN)){
-    SetMessage(IllegalCharInMailAddr);
-    return FALSE;
+  if(*mailAddress!='\0'){
+    if(!IsSafeString(mailAddress, BUFFMAXLN)){
+      SetMessage(IllegalCharInMailAddr);
+      return FALSE;
+    }
   }
 
   /* if illegal mac address, return */
@@ -388,7 +441,7 @@ int isSafeString(char* str, int length){
 /*********************************************/
 /* put auth request page to client            */
 /*********************************************/
-int putAuthRequestPageToClient(char *language, char* cgiName)
+int putAuthRequestPageToClient(char *language, char* cgiName, char* docName, char* redirectedUrl)
 {
   char authdoc[BUFFMAXLN];
   FILE *fp;
@@ -396,7 +449,7 @@ int putAuthRequestPageToClient(char *language, char* cgiName)
 
   /* make read in path to the retry document */
   snprintf(authdoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	  GetConfValue("OpengateDir"),language,GetConfValue("AuthDoc"));
+	  GetConfValue("OpengateDir"),language,docName);
 
   /* replace keyword and send out the file */
   printf("Content-type: text/html\r\n\r\n");
@@ -408,6 +461,7 @@ int putAuthRequestPageToClient(char *language, char* cgiName)
 
   while(fgets(buff,BUFFMAXLN,fp)!=NULL){
     HtmlReplace(buff, "%%CGINAME%%", cgiName);
+    HtmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
     if(strstr(buff, "%%ERRORLIST%%")!=NULL){
       InsertMessageToPage(language);
     }else{
@@ -508,19 +562,37 @@ int putCheckPageToClient(char *language, char* userId, char* extraId)
 }
 /*********************************************/
 /* put response to client for register request */
+/* ownReg:1=the page is controlled by owner, 0=under admin */
 /*********************************************/
-int putRegisterPageToClient(char *language, char* macAddress, char* deviceName, char* mailAddress, char* userId, char* extraId)
+int putRegisterPageToClient(char *language, char* macAddress, char* deviceName, char* mailAddress, char* userId, char* extraId, int ownReg, char* redirectedUrl)
 {
   char responsedoc[BUFFMAXLN];
   FILE *fp;
   char buff[BUFFMAXLN];
   char cookie[SIDMAXLN];
-  char* chkCgi=GetConfValue("CheckCgi");
-  char* regCgi=GetConfValue("RegisterCgi");
+  char* regCgi="";
+  char* regDoc="";
+  char* maxDevicesStr;
+
+  /* get Maximum count of devices for an user from conf file */
+  maxDevicesStr=GetConfValue("MaxDevices");
+
+  /* setup cgi and doc */
+  if(ownReg){
+    regCgi=GetConfValue("OwnCgi");
+    regDoc=GetConfValue("OwnRegisterDoc");
+  }else{
+    regCgi=GetConfValue("RegisterCgi");
+    regDoc=GetConfValue("RegisterDoc");
+  }
+  if(isNull(regCgi) || isNull(regDoc)){
+    err_msg("ERR at %s#%d: cannot find cgi/doc for reg in conf",__FILE__,__LINE__);
+    return FALSE;
+  }
 
   /* make read in path to the document */
   snprintf(responsedoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	  GetConfValue("OpengateDir"),language,GetConfValue("RegisterDoc"));
+	  GetConfValue("OpengateDir"),language,regDoc);
 
   /* send header */
   printf("Content-type: text/html\r\n");
@@ -542,12 +614,13 @@ int putRegisterPageToClient(char *language, char* macAddress, char* deviceName,
   }
 
   while(fgets(buff,BUFFMAXLN,fp)!=NULL){
-    HtmlReplace(buff, "%%CHECKCGI%%", chkCgi);
     HtmlReplace(buff, "%%REGISTERCGI%%", regCgi);
     HtmlReplace(buff, "%%MACADDR%%", macAddress);
     HtmlReplace(buff, "%%DEVICE%%", deviceName);
     HtmlReplace(buff, "%%MAILADDR%%", mailAddress);
     HtmlReplace(buff, "%%USERID%%", userId);
+    HtmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
+    HtmlReplace(buff, "%%MAXDEVICES%%", maxDevicesStr);
 
     if(strstr(buff, "%%MACREGLIST%%")!=NULL){
       PutMacRegListToClient(userId, extraId);
@@ -660,7 +733,40 @@ void putMessageToClient(char *message)
   printf("<BODY>\r\n");
   printf("%s\r\n",     message);
   printf("</BODY></HTML> \r\n\r\n");
+}
 
+/************************************************/
+/* send page for returning to the previous page */
+/************************************************/
+void returnToRedirectedPage(char* redirectedUrl, char* language){
+  
+  char returndoc[BUFFMAXLN];
+  FILE *fp;
+  char buff[BUFFMAXLN];
+  char* waitTime="10";
+ 
+  /* make read in path to the document */
+  snprintf(returndoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
+	  GetConfValue("OpengateDir"),language,GetConfValue("ReturnDoc"));
+
+  /* send header */
+  printf("Content-type: text/html\r\n\r\n");
+  
+  if((fp=fopen(returndoc, "r"))==NULL){
+    err_msg("ERR at %s#%d: cannot find file %s",__FILE__,__LINE__, returndoc);
+    printf("Error! \r\n\r\n");
+    return;
+  }
+
+  if(!isNull(GetConfValue("ReturnWaitTime"))){
+    waitTime=GetConfValue("ReturnWaitTime");
+  }
+  while(fgets(buff,BUFFMAXLN,fp)!=NULL){
+    HtmlReplace(buff, "%%WAITTIME%%", waitTime);
+    HtmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
+    printf("%s",buff);    
+  }
+  fclose(fp);
 }
 
 /************************************/
@@ -735,16 +841,16 @@ int htmlReplace(char* buff,char *beforeStr,char *afterStr)
   
   if(buff==NULL) return 1;
 
-  strncpy(tempBuff, buff, BUFFMAXLN);
-  strncpy(buff,"",BUFFMAXLN);
+  strlcpy(tempBuff, buff, BUFFMAXLN);
+  strlcpy(buff,"",BUFFMAXLN);
   
   for(pBuff = tempBuff;
       (pNext=StrSplit(pBuff, beforeStr)) != NULL;
       pBuff = pNext){
-    strncat(buff,pBuff,BUFFMAXLN);
-    strncat(buff,afterStr,BUFFMAXLN);
+    strlcat(buff,pBuff,BUFFMAXLN);
+    strlcat(buff,afterStr,BUFFMAXLN);
   }
-  strncat(buff,pBuff,BUFFMAXLN);
+  strlcat(buff,pBuff,BUFFMAXLN);
   
   return 0;
 }
@@ -787,7 +893,7 @@ int getHttpCookie(char *cookie, char* cookieName){
 
  /* if exist cookie, copy it to work area */
   if(isNull(getenv("HTTP_COOKIE"))) return FALSE;
-  strncpy(content, getenv("HTTP_COOKIE"), BUFFMAXLN);
+  strlcpy(content, getenv("HTTP_COOKIE"), BUFFMAXLN);
   ptr=content;
 
   /* search 'OpengateMmng' cookie string (terminated by ; or \0) */
@@ -805,7 +911,7 @@ int getHttpCookie(char *cookie, char* cookieName){
     split(ptr, name, value, next);
 
     if(strstr(name[0], cookieName)!=NULL){
-      strncpy(cookie, value[0], SIDMAXLN);
+      strlcpy(cookie, value[0], SIDMAXLN);
     }
     ptr=next[0];
   }
@@ -848,14 +954,22 @@ int analyzeUpdateRequestAndExecute(char *requestStr, char* userId, char* extraId
   int modified=FALSE;    /* database modification is executed */
   int ret;
   char content[BUFFMAXLN];
+  char deviceName[WORDMAXLN]="";
+  char mailAddress[BUFFMAXLN]="";
 
   /* if null string, return */
   if(isNull(requestStr)) return FALSE;
 
   /* copy request string to work area */
-  strncpy(content, requestStr, BUFFMAXLN);
+  strlcpy(content, requestStr, BUFFMAXLN);
 
   /* split request item and execute the request */
+  /* <input type=radio name=11:22:33:44:55:66 value=extend> */
+  /* <input type=radio name=11:22:33:44:55:66 value=pause> */
+  /* <input type=radio name=11:22:33:44:55:66 value=delete> */
+  /* <input type=hidden name=11:22:33:44:55:66 value=mail:wata@foo.bar> */
+  /* <input type=hidden name=11:22:33:44:55:66 value=name:iPhone01> */
+
   ptr=content;
   while(ptr!=NULL){
 
@@ -863,43 +977,96 @@ int analyzeUpdateRequestAndExecute(char *requestStr, char* userId, char* extraId
     split(ptr, name, value, next);
     
     /* if item=delete, execute delete */
-    if(strcmp(name[0], "delete")==0){
-      strncpy(macAddr, value[0], ADDRMAXLN);
+    if(strcmp(value[0], "delete")==0){
+      strlcpy(macAddr, name[0], ADDRMAXLN);
       decode(macAddr);
       if(ConvertMacAddr(macAddr)){
 	ret=DelMacAddrFromMngDb(macAddr);
 	if(ret){
 	  modified=TRUE;
 	  PutMacModifyLogToMngDb(userId, extraId, macAddr, 'D');
+	  PutMacAddressToServers(macAddr);
 	}
       }
     }
 
     /* if item = extend, execute extend */
-    else if(strcmp(name[0], "extend")==0){
-      strncpy(macAddr, value[0], ADDRMAXLN);
+    else if(strcmp(value[0], "extend")==0){
+      strlcpy(macAddr, name[0], ADDRMAXLN);
       decode(macAddr);
       if(ConvertMacAddr(macAddr)){
 	ret=RenewMacAddrInMngDb(macAddr);
 	if(ret){
 	  modified=TRUE;
 	  PutMacModifyLogToMngDb(userId, extraId, macAddr, 'E');
+	  PutMacAddressToServers(macAddr);
 	}
       }
     }
 
     /* if item = pause, execute pause */
-    else if(strcmp(name[0], "pause")==0){
-      strncpy(macAddr, value[0], ADDRMAXLN);
+    else if(strcmp(value[0], "pause")==0){
+      strlcpy(macAddr, name[0], ADDRMAXLN);
       decode(macAddr);
       if(ConvertMacAddr(macAddr)){
 	ret=PauseMacAddrInMngDb(macAddr);
 	if(ret){
 	  modified=TRUE;
 	  PutMacModifyLogToMngDb(userId, extraId, macAddr, 'P');
+	  PutMacAddressToServers(macAddr);
 	}
       }
     }
+
+    /* if item = name xxxx, execute renaming the device name */
+    else if(strstr(value[0], "name+")==value[0]){
+      strlcpy(macAddr, name[0], ADDRMAXLN);
+      decode(macAddr);
+      if(ConvertMacAddr(macAddr)){
+
+	/* when value=name xxxx, rename device-name to xxxx */
+	/* copy limitted length and decode */
+	strlcpy(deviceName, value[0]+5, WORDMAXLN);
+	decode(deviceName);
+
+	/* if illegal device name, return */
+	if(*deviceName=='\0'){
+	  SetMessage(EmptyDeviceName); 
+	  return FALSE;
+	}
+	if(!IsSafeString(deviceName, WORDMAXLN)){
+	  SetMessage(IllegalCharInDevice);
+	  return FALSE;
+	}
+
+	/* update the device name in db */
+	RenameDeviceNameInMngDb(macAddr, deviceName);
+      }
+    }
+
+    /* if item = mail xxxx, execute renaming the mail address */
+    else if(strstr(value[0], "mail+")==value[0]){
+      strlcpy(macAddr, name[0], ADDRMAXLN);
+      decode(macAddr);
+      if(ConvertMacAddr(macAddr)){
+
+	/* when value=mail xxxx, rename mail-address to xxxx */
+	/* copy limitted length and decode */
+	strlcpy(mailAddress, value[0]+5, BUFFMAXLN);
+	decode(mailAddress);
+
+	/* if illegal mail address, return */
+	if(*mailAddress!='\0'){
+	  if(!IsSafeString(mailAddress, BUFFMAXLN)){
+	    SetMessage(IllegalCharInMailAddr);
+	    return FALSE;
+	  }
+	}
+	
+	/* update the device name in db */
+	RenameMailAddressInMngDb(macAddr, mailAddress);
+      }
+    }
     
     /* shift pointer to next item */
     ptr=next[0];
@@ -908,20 +1075,36 @@ int analyzeUpdateRequestAndExecute(char *requestStr, char* userId, char* extraId
   return modified;
 }
 
-
 /*********************************************/
 /* put response to client            */
+/* ownUpdate:1=page is controlled by owner, 0=under admin */
 /*********************************************/
-int putUpdatePageToClient(char *language, char* userId, char* extraId)
+int putUpdatePageToClient(char *language, char* userId, char* extraId, int ownUpdate, char* redirectedUrl)
 {
   char responsedoc[BUFFMAXLN];
   FILE *fp;
   char buff[BUFFMAXLN];
   char cookie[SIDMAXLN];
+  char* updateCgi="";
+  char* updateDoc="";
+  char mailDefault[BUFFMAXLN];
+
+  /* select update page for owner or administrator */
+  if(ownUpdate){
+    updateCgi=GetConfValue("OwnCgi");
+    updateDoc=GetConfValue("OwnUpdateDoc");
+  }else{
+    updateCgi=GetConfValue("UpdateCgi");
+    updateDoc=GetConfValue("UpdateDoc");
+  }
+  if(isNull(updateCgi) || isNull(updateDoc)){
+    err_msg("ERR at %s#%d: cannot find cgi/doc for update in conf",__FILE__,__LINE__);
+    return FALSE;
+  }
 
   /* make read in path to the retry document */
   snprintf(responsedoc, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
-	  GetConfValue("OpengateDir"),language,GetConfValue("UpdateDoc"));
+	  GetConfValue("OpengateDir"),language,updateDoc);
 
   /* send header */
   printf("Content-type: text/html\r\n");
@@ -931,6 +1114,8 @@ int putUpdatePageToClient(char *language, char* userId, char* extraId)
     CreateCookie(cookie);
     printf("Set-Cookie: %s=%s;path=/;\r\n", GetConfValue("AuthUserCookie"), cookie);
     SaveCookieToWorkDb(cookie,userId, extraId, NORMALUSER);
+    MakeMailDefault(userId, extraId, mailDefault);
+    SaveMailDefalutForCookieToWorkDb(cookie, mailDefault);
   }
 
   /* end of http header */
@@ -942,8 +1127,10 @@ int putUpdatePageToClient(char *language, char* userId, char* extraId)
   }
 
   while(fgets(buff,BUFFMAXLN,fp)!=NULL){
-    HtmlReplace(buff, "%%CGINAME%%", GetConfValue("UpdateCgi"));
+
+    HtmlReplace(buff, "%%CGINAME%%", updateCgi);
     HtmlReplace(buff, "%%USERID%%", userId);
+    HtmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
 
     if(strstr(buff, "%%MACLIST%%")!=NULL){
       PutMacListToClient(userId,extraId);
@@ -962,6 +1149,7 @@ int putUpdatePageToClient(char *language, char* userId, char* extraId)
   return TRUE;
 }
 
+
 /*********************************************/
 /* put mac list the client            */
 /*********************************************/
@@ -979,17 +1167,21 @@ void putMacListToClient(char* userId, char* extraId){
   while(GetNextMacAddrFromMngDb(userId,extraId,macAddr,deviceName,
 				entryDate,limitDate,status,mailAddress)){
     printf("<tr align=middle>\n");
-    if(status[0]=='A'){
-      printf("<td><input type=checkbox name=extend value=%s></td>\n",macAddr);
-      printf("<td><input type=checkbox name=pause value=%s></td>\n",macAddr);
-      printf("<td><input type=checkbox name=delete value=%s></td>\n",macAddr);
+    if((status[0]=='A')||(status[0]=='P')){
+      printf("<td><input type=radio name=%s value=extend checked></td>\n",
+	     macAddr);
+      printf("<td><input type=radio name=%s value=pause></td>\n",macAddr);
+      printf("<td><input type=radio name=%s value=delete></td>\n",macAddr);
     }else{
       printf("<td>-</td>\n");
       printf("<td>-</td>\n");
       printf("<td>-</td>\n");
     }
     printf("<td>%s</td>\n", macAddr);
-    printf("<td>%s</td>\n",deviceName);
+    printf("<td onclick=\"editstring('%s','name',this)\">%s</td>\n",
+	   macAddr,deviceName);
+    printf("<td onclick=\"editstring('%s','mail',this)\">%s</td>\n",
+	   macAddr,mailAddress);
     printf("<td>%s</td>\n",entryDate);
     printf("<td>%s</td>\n",limitDate);
     printf("<td>%s</td>\n",status);
@@ -1002,32 +1194,85 @@ void putMacListToClient(char* userId, char* extraId){
 /*********************************************/
 void putUsageLogToClient(char *userId, char* extraId, char* language){
 
-  char macAddr[ADDRMAXLN];
-  char deviceName[WORDMAXLN];
-  char openTime[WORDMAXLN];
-  char gatewayName[WORDMAXLN];
-  int weekday;
-  char str[WORDMAXLN];
+  char macAddr[ADDRMAXLN]="";
+  char deviceName[WORDMAXLN]="";
+  char openTime[WORDMAXLN]="";
+  char closeTime[WORDMAXLN]="";
+  char gatewayName[WORDMAXLN]="";
+  char beginTime[WORDMAXLN]="";
+  char endTime[WORDMAXLN]="";
+  int dateCount=0;
   char* p;
+  int count=0;
 
-  /* get usage log form db and insert as html table row */
+  /* get usage log from db and insert as JSON format */
   while(GetNextUsageLogFromMngDb(userId,extraId,macAddr,deviceName,
-				 openTime,gatewayName,&weekday)){
+				 openTime,closeTime,gatewayName)){
 
     /* split hostname only */
     if((p=strchr(gatewayName,'.'))!=NULL) *p='\0';
 
     /* put out table row */
-    printf("<tr align=middle>\n");
-    printf("<td>%s</td>\n", macAddr);
-    printf("<td>%s</td>\n",deviceName);
-    printf("<td>%s</td>\n",openTime);
-    printf("<td>%s</td>\n",WeekdayStr(weekday, str, language));
-    printf("<td>%s</td>\n",gatewayName);
-    printf("</tr>\n");
+    printf("\t %d:{\n", count);
+    printf("\t\t macAddress: '%s',\n", macAddr);
+    printf("\t\t device: '%s',\n",deviceName);
+    printf("\t\t openTime: '%s',\n",openTime);
+    printf("\t\t closeTime: '%s',\n",closeTime);
+    printf("\t\t gateway: '%s'\n",gatewayName);
+    printf("\t },\n");
+
+    /* increment log item counter */
+    count++;
   }
+
+  /* put out informations for the list */
+  GetTimeRangeToShowLog(beginTime, endTime, &dateCount);
+  printf("\t info:{\n");
+  printf("\t\t beginTime: '%s',\n", beginTime);
+  printf("\t\t endTime: '%s',\n", endTime);
+  printf("\t\t listCount: %d,\n", count);
+  printf("\t\t dateCount: %d\n", dateCount);
+  printf("\t }\n");
 }
 
+/**********************************************/
+/* check allowable http-agent defined in conf */
+/**********************************************/
+int isAccessedFromAllowableAgent(void){
+
+  char* pAgent=NULL;
+  char* pRegExPattern=NULL;
+  int found=FALSE;
+
+  /* get agent string. if not, return false */
+  if( isNull(pAgent=getenv("HTTP_USER_AGENT")) ) return FALSE;
+
+  /* get first reg expression in conf. if not, return true(not define=allow all) */
+  if(isNull(pRegExPattern=GetFirstConfValue("AllowableAgentPattern"))){
+    return TRUE;
+  }
+
+  /* loop for patterns in conf */
+  while(!isNull(pRegExPattern)){
+
+    /* if agent is matched to reg ex, return true. */
+    /* last-arg 1 means case sensitive */
+    if(RegExMatch(pAgent, pRegExPattern, 1)){
+      found=TRUE;
+      break;
+    }
+
+    /* get next reg expression in conf */
+    pRegExPattern=GetNextConfValue();
+  }
+
+  /* if fail, print message */
+  if(!found){
+    err_msg("ERR at %s#%d: http-agent[%s] is not allowed in conf file",__FILE__,__LINE__, pAgent);
+  }
+
+  return found;
+}
 
 /*******************************/
 /*******************************/
@@ -1036,7 +1281,7 @@ int GetPostData(char *content, int contentMaxLength){
   int ret;
   if(debug>1) err_msg("DEBUG:=>getPostData(%d)", contentMaxLength);
   ret=getPostData(content, contentMaxLength);
-  if(debug>1) err_msg("DEBUG:%d<=getPostData(%s)",ret, content);
+  if(debug>1) err_msg("DEBUG:%d<=getPostData(..)",ret);
   return ret;
 }
 
@@ -1047,6 +1292,15 @@ int GetLangFromQueryString(char* language){
   if(debug>1) err_msg("DEBUG:%d<=getLangFromQueryString(%s)",ret,language);
   return ret;
 }
+
+int GetRedirectedUrlFromQueryString(char* redirectedUrl){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>getRedirectedUrlFromQueryString( )");
+  ret=getRedirectedUrlFromQueryString(redirectedUrl);
+  if(debug>1) err_msg("DEBUG:%d<=getRedirectedUrlFromQueryString(%s)",ret,redirectedUrl);
+  return ret;
+}
+
 int GetMacAddrFromQueryString(char* macAddress){
   int ret;
   if(debug>1) err_msg("DEBUG:=>getMacAddrFromQueryString( )");
@@ -1065,9 +1319,9 @@ int AnalyzeCheckRequest(char *content, int* status, char* macAddress){
 
 int AnalyzeRegisterRequest(char *content, char* macAddress, char* deviceName, char* mailAddress){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>analyzeRegisterRequestAndExecute(%s)", content);
+  if(debug>1) err_msg("DEBUG:=>analyzeRegisterRequest(%s)", content);
   ret=analyzeRegisterRequest(content, macAddress, deviceName, mailAddress);
-  if(debug>1) err_msg("DEBUG:%d<=analyzeRegisterRequestAndExecute(%s,%s,%s)",ret, macAddress, deviceName, mailAddress);
+  if(debug>1) err_msg("DEBUG:%d<=analyzeRegisterRequest(%s,%s,%s)",ret, macAddress, deviceName, mailAddress);
   return ret;
 }
 
@@ -1088,11 +1342,10 @@ int PutCheckPageToClient(char *language, char* userId, char* extraId){
   return ret;
 }
 
-int PutRegisterPageToClient(char *language, char* macAddress, char* deviceName, char* mailAddress, char* userId, char* extraId){
+int PutRegisterPageToClient(char *language, char* macAddress, char* deviceName, char* mailAddress, char* userId, char* extraId, int ownReg, char* redirectedUrl){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>putRegisterPageToClient(%s,%s,%s,%s,%s,%s,%s)", 
-		      language,macAddress,deviceName,mailAddress,userId,extraId);
-  ret=putRegisterPageToClient(language, macAddress, deviceName, mailAddress, userId, extraId);
+  if(debug>1) err_msg("DEBUG:=>putRegisterPageToClient(%s,%s,%s,%s,%s,%s,%d,%s)", language,macAddress,deviceName,mailAddress,userId,extraId,ownReg, redirectedUrl);
+  ret=putRegisterPageToClient(language, macAddress, deviceName, mailAddress, userId, extraId, ownReg, redirectedUrl);
   if(debug>1) err_msg("DEBUG:(%d)<=putRegisterPageToClient( )",ret);
   return ret;
 }
@@ -1115,6 +1368,12 @@ void PutMessageToClient(char *message){
   if(debug>1) err_msg("DEBUG:<=putMessageToClient( )");
 }
 
+void ReturnToRedirectedPage(char* redirectedUrl, char* language){
+  if(debug>1) err_msg("DEBUG:=>returnToRedirectedPage(%s,%s)",redirectedUrl,language);
+  returnToRedirectedPage(redirectedUrl, language);
+  if(debug>1) err_msg("DEBUG:<=returnToRedirectedPage( )");
+}
+
 int ConvertMacAddr(char* macAddr){
   int ret;
   if(debug>1) err_msg("DEBUG:=>convertMacAddr(%s)", macAddr);
@@ -1173,16 +1432,16 @@ int GetUserIdFromEnv(char *userid){
 
 int GetUserIdFromPostData(char* requestStr, char* userid, char* password){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>getUserIdFromPostData(%s)",requestStr);
+  if(debug>1) err_msg("DEBUG:=>getUserIdFromPostData(..)");
   ret = getUserIdFromPostData(requestStr,userid,password);
   if(debug>1) err_msg("DEBUG:(%d)<=getUserIdFromPostData(,%s,password)",ret,userid);
   return ret;
 }
 
-int PutAuthRequestPageToClient(char *language, char* cgiName){
+int PutAuthRequestPageToClient(char *language, char* cgiName, char* docName, char* redirectedUrl){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>putAuthRequestPageToClient(%s,%s)",language,cgiName);
-  ret=putAuthRequestPageToClient(language,cgiName);
+  if(debug>1) err_msg("DEBUG:=>putAuthRequestPageToClient(%s,%s,%s)",language,cgiName, docName, redirectedUrl);
+  ret=putAuthRequestPageToClient(language,cgiName, docName, redirectedUrl);
   if(debug>1) err_msg("DEBUG:(%d)<=putAuthRequestPageToClient( )",ret);
   return ret;
 }
@@ -1196,11 +1455,11 @@ int AnalyzeUpdateRequestAndExecute(char *content, char* userId, char* extraId){
   return ret;
 }
 
-int PutUpdatePageToClient(char *language, char* userId, char* extraId){
+int PutUpdatePageToClient(char *language, char* userId, char* extraId, int ownUpdate, char* redirectedUrl){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>putUpdatePageToClient(%s,%s,%s)", 
-		      language,userId,extraId);
-  ret=putUpdatePageToClient(language,userId,extraId);
+  if(debug>1) err_msg("DEBUG:=>putUpdatePageToClient(%s,%s,%s,%d,%s)", 
+		      language,userId,extraId,ownUpdate,redirectedUrl);
+  ret=putUpdatePageToClient(language,userId,extraId,ownUpdate,redirectedUrl);
   if(debug>1) err_msg("DEBUG:(%d)<=putUpdatePageToClient( )",ret);
   return ret;
 }
@@ -1216,3 +1475,11 @@ void PutUsageLogToClient(char *userId, char* extraId, char* language){
   putUsageLogToClient(userId,extraId,language);
   if(debug>1) err_msg("DEBUG:<=putUsageLogToClient( )");
 }
+
+int IsAccessedFromAllowableAgent(void){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>isAccessedFromAllowableAgent( )");
+  ret=isAccessedFromAllowableAgent();
+  if(debug>1) err_msg("DEBUG:(%d)<=isAccessedFromAllowableAgent( )",ret);
+  return ret;
+}
