@@ -1,5 +1,5 @@
 /**************************************************
-opengate Mac addr auth program
+OpengateM - MAC address authentication system 
 
  module to control management database
  list of mac addresses
@@ -28,7 +28,6 @@ Email: watanaby@is.saga-u.ac.jp
 #include "opengatemmng.h"
 #include <mysql.h>
 
-
 MYSQL mysql;
 
 /******************************************
@@ -91,7 +90,7 @@ int getNicVendorFromMngDb(char* macAddress, char* vendor, int bufferLength){
   vendor[1]='\0';
  
   /* prepare query string */
-  strncpy(macHeader, macAddress, ADDRMAXLN);
+  strlcpy(macHeader, macAddress, ADDRMAXLN);
   macHeader[8]='\0';
   snprintf(queryStr, BUFFMAXLN, 
 	   "select org from nicvendors where oui='%s'",
@@ -114,7 +113,7 @@ int getNicVendorFromMngDb(char* macAddress, char* vendor, int bufferLength){
 
   /* if found */
   else {
-    strncpy(vendor, row[0], bufferLength);
+    strlcpy(vendor, row[0], bufferLength);
     found=TRUE;
   }
 
@@ -193,7 +192,7 @@ int countMacAddrsForUserInMngDb(char* userId, char* extraId){
 
   /* if found, return values */
   if(row!=NULL){
-    strncpy(countStr, row[0], WORDMAXLN);
+    strlcpy(countStr, row[0], WORDMAXLN);
     count=atoi(countStr);
   }
 
@@ -282,12 +281,12 @@ int getNextMacAddrFromMngDb(char* userId, char* extraId, char* macAddress, char*
 
   /* if found, return values */
   if(row!=NULL){
-    strncpy(macAddress, row[0],ADDRMAXLN);
-    strncpy(deviceName,row[1],WORDMAXLN);
-    strncpy(entryDate,row[2],WORDMAXLN);
-    strncpy(limitDate,row[3],WORDMAXLN);
-    strncpy(status,row[4],WORDMAXLN);
-    strncpy(mailAddress,row[5],BUFFMAXLN);
+    strlcpy(macAddress, row[0],ADDRMAXLN);
+    strlcpy(deviceName,row[1],WORDMAXLN);
+    strlcpy(entryDate,row[2],WORDMAXLN);
+    strlcpy(limitDate,row[3],WORDMAXLN);
+    strlcpy(status,row[4],WORDMAXLN);
+    strlcpy(mailAddress,row[5],BUFFMAXLN);
     return TRUE;
   }
   /* if not found, free memory area */
@@ -325,7 +324,7 @@ int putMacModifyLogToMngDb(char* userId, char* extraId, char* macAddr, char modi
 /******************************************
  The count of mac address modification in last 24 hours
 ******************************************/
-int countMacModifyPerDayInMngDb(char* userId, char* extraId){
+int countMacModifyPerDayInMngDb(char* userId, char* extraId, char* macAddress){
 
   MYSQL_RES *res=NULL;
   MYSQL_ROW row;
@@ -334,12 +333,21 @@ int countMacModifyPerDayInMngDb(char* userId, char* extraId){
   int count=10000;
 
   /* prepare query string */
-  snprintf(queryStr, BUFFMAXLN, 
+  /* count for one macAddress if not null, elsecount for one userID)*/
+  if(!isNull(macAddress)){
+    snprintf(queryStr, BUFFMAXLN, 
+	   "select count(*) from macmodify "
+	   " where macAddress='%s' and "
+	   " modifyDate > adddate(now(), interval -1 day) ",
+	   macAddress);
+  }else{
+    snprintf(queryStr, BUFFMAXLN, 
 	   "select count(*) from macmodify "
 	   " where userId='%s' and extraId='%s' and "
 	   " modifyDate > adddate(now(), interval -1 day) ",
 	   userId, extraId);
-
+  }
+  
   /* send SQL query */
   if (mysql_query(&mysql, queryStr)){
     err_msg("ERR at %s#%d: mysql query: %s",__FILE__,__LINE__,
@@ -355,7 +363,7 @@ int countMacModifyPerDayInMngDb(char* userId, char* extraId){
 
   /* if found, return values */
   if(row!=NULL){
-    strncpy(countStr, row[0], WORDMAXLN);
+    strlcpy(countStr, row[0], WORDMAXLN);
     count=atoi(countStr);
   }
 
@@ -381,7 +389,7 @@ int delMacAddrFromMngDb(char* macAddr){
     /* don't touch device set as status=Inactive */
     snprintf(queryStr, BUFFMAXLN, 
 	     "update macaddrs set status='D',limitDate=now() "
-	     " where macAddress='%s' and status='A'", macAddr);
+	     " where macAddress='%s' and (status='A' or status='P')", macAddr);
   }else{
   
     /* prepare query string */
@@ -389,7 +397,7 @@ int delMacAddrFromMngDb(char* macAddr){
     /* don't touch device set as status=Inactive */
     snprintf(queryStr, BUFFMAXLN, 
 	     "delete from macaddrs "
-	     " where macAddress='%s' and status='A'", macAddr);
+	     " where macAddress='%s' and (status='A' or status='P')", macAddr);
   }
 
   /* send SQL query */
@@ -412,8 +420,8 @@ int renewMacAddrInMngDb(char* macAddr){
   /* prepare query string */
   /* don't touch device set as inactive by admin */
   snprintf(queryStr, BUFFMAXLN, 
-	   "update macaddrs set renewDate=now(), limitDate=%s "
-	   " where status='A' and macAddress='%s'", 
+	   "update macaddrs set status='A', renewDate=now(), limitDate=%s "
+	   " where (status='A' or status='P') and macAddress='%s'", 
 	   GetConfValue("LimitDate"),macAddr);
 
   /* send SQL query */
@@ -428,7 +436,6 @@ int renewMacAddrInMngDb(char* macAddr){
 /******************************************
 pause the usage of the device for a mac address
  registered in the management db
- (set the limitDate to now)
 ******************************************/
 int pauseMacAddrInMngDb(char* macAddr){
 
@@ -437,7 +444,7 @@ int pauseMacAddrInMngDb(char* macAddr){
   /* prepare query string */
   /* don't touch device set as inactive by admin */
   snprintf(queryStr, BUFFMAXLN, 
-	   "update macaddrs set limitDate=now() "
+	   "update macaddrs set status='P' "
 	   " where status='A' and macAddress='%s' and limitDate>now()", 
 	   macAddr);
 
@@ -496,7 +503,7 @@ int isSessionFoundInMngDb(char* macAddr){
  get next next usage log from management db
  if end of list, return false
 *******************************************/
-int getNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* deviceName, char* openTime, char* gatewayName, int* weekday){
+int getNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* deviceName, char* openTime, char* closeTime, char* gatewayName){
   static MYSQL_RES *res=NULL;
   MYSQL_ROW row;
   char queryStr[BUFFMAXLN];
@@ -509,15 +516,15 @@ int getNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* d
 
     /* prepare query string */
     /* get log where addr.entry < session.open < addr.limit */
+    /*  (the same device may be registered by other users in old days) */
     snprintf(queryStr, BUFFMAXLN, 
-	     "select distinct macaddrs.macAddress, device, date(openTime), "
-	     " gatewayName, weekday(openTime) "
+	     "select macaddrs.macAddress, device, openTime, closeTime, "
+	     " gatewayName "
 	     " from macaddrs, sessionmd "
 	     " where macaddrs.macAddress=sessionmd.macAddress "
 	     " and entryDate < openTime and openTime < limitDate "
-	     " and %s < openTime "
-	     " and userId='%s' and extraId='%s' and status!='D'"
-	     " order by openTime desc",
+	     " and (%s<closeTime or closeTime=0) "
+	     " and userId='%s' and extraId='%s'",
 	     GetConfValue("ShowLogAfter"), userId, extraId);
 
     /* send SQL query */
@@ -536,11 +543,11 @@ int getNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* d
 
   /* if found, return values */
   if(row!=NULL){
-    strncpy(macAddr, row[0],ADDRMAXLN);
-    strncpy(deviceName,row[1],WORDMAXLN);
-    strncpy(openTime,row[2],WORDMAXLN);
-    strncpy(gatewayName,row[3],WORDMAXLN);
-    *weekday=atoi(row[4]);
+    strlcpy(macAddr, row[0],ADDRMAXLN);
+    strlcpy(deviceName,row[1],WORDMAXLN);
+    strlcpy(openTime,row[2],WORDMAXLN);
+    strlcpy(closeTime,row[3],WORDMAXLN);
+    strlcpy(gatewayName,row[4],WORDMAXLN);
     return TRUE;
   }
   /* if not found, free memory area */
@@ -550,6 +557,7 @@ int getNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* d
   }
 }
 
+
 /*******************************************
  get next mail address near limit date
 from management db 
@@ -575,10 +583,12 @@ int getNextMailAddressFromMngDb(char* mailAddress, char* limitDate, char* device
     /* [select mailAddress,limitDate,device from macaddrs 
         where (  date(now())=date(adddate(limitDate, interval -1 day)) 
               OR date(now())=date(adddate(limitDate, interval -7 day))  ) 
-             AND status='A'] */
+             AND (status='A' OR status='P') order by mailAddress,limitDate] */
     snprintf(queryStr, BUFFMAXLN, 
 	     "select mailAddress,limitDate,device from macaddrs "
-	     " where (%s) and status='A'", mailTiming);
+	     " where (%s) and (status='A' or status='P') "
+	     " order by mailAddress,limitDate", mailTiming);
+    /*       mailTiming is a condition string defined in conf file */ 
 
     /* send SQL query */
     if (mysql_query(&mysql, queryStr)){
@@ -596,9 +606,69 @@ int getNextMailAddressFromMngDb(char* mailAddress, char* limitDate, char* device
 
   /* if found, return values */
   if(row!=NULL){
-    strncpy(mailAddress, row[0], ADDRMAXLN);
-    strncpy(limitDate, row[1], WORDMAXLN);
-    strncpy(device, row[2], WORDMAXLN);
+    strlcpy(mailAddress, row[0], BUFFMAXLN);
+    strlcpy(limitDate, row[1], WORDMAXLN);
+    strlcpy(device, row[2], WORDMAXLN);
+    return TRUE;
+  }
+  /* if not found, free memory area */
+  else{
+    mysql_free_result(res);
+    return FALSE;
+  }
+}
+
+/*******************************************
+ get the date corresponding to ShowLogAfter in conf file
+*******************************************/
+int getTimeRangeToShowLog(char* beginTime, char* endTime, int* dateCount){
+
+  static MYSQL_RES *res=NULL;
+  MYSQL_ROW row;
+  char queryStr[BUFFMAXLN];
+  char* showLogAfter;
+  char countStr[WORDMAXLN];
+
+  /* get conf value for beginning day for listing */
+  showLogAfter=GetConfValue("ShowLogAfter");
+  if(isNull(showLogAfter)) return FALSE;
+
+  /* set default values */
+  beginTime[0]='\0';
+  endTime[0]='\0';
+  *dateCount=0;
+
+  /* if do not get result yet */
+  if(res==NULL){
+
+    /* prepare query string */
+    /* select (adddate(now(), interval -1 month)), 
+       now(), 
+       datediff(now(),(adddate(now(), interval -1 month))) + 1; */
+    snprintf(queryStr, BUFFMAXLN, 
+	     "select (%s), now(), datediff(now(),(%s)) + 1", 
+	     showLogAfter, showLogAfter);
+
+    /* send SQL query */
+    if (mysql_query(&mysql, queryStr)){
+      err_msg("ERR at %s#%d: mysql query[%s]: %s",__FILE__,__LINE__,queryStr,
+	      mysql_error(&mysql));
+      return FALSE;
+    }
+    
+    /* store result */
+    res = mysql_store_result(&mysql);
+  }
+
+  /* output table name */
+  row = mysql_fetch_row(res);
+
+  /* if found, return values */
+  if(row!=NULL){
+    strlcpy(beginTime, row[0], WORDMAXLN);
+    strlcpy(endTime, row[1], WORDMAXLN);
+    strlcpy(countStr, row[2], WORDMAXLN);
+    *dateCount=atoi(countStr);
     return TRUE;
   }
   /* if not found, free memory area */
@@ -608,6 +678,117 @@ int getNextMailAddressFromMngDb(char* mailAddress, char* limitDate, char* device
   }
 }
 
+/********************************************
+rename the device name for indicated mac address
+********************************************/
+int renameDeviceNameInMngDb(char* macAddr, char* nameStr){
+
+  char queryStr[BUFFMAXLN];
+
+  /* prepare query string */
+  snprintf(queryStr, BUFFMAXLN, 
+	   "update macaddrs set device='%s' "
+	   " where (status='A' or status='P') and macAddress='%s'", 
+	   nameStr, macAddr);
+
+  /* send SQL query */
+  if (mysql_query(&mysql, queryStr)){
+    err_msg("ERR at %s#%d: mysql query: %s",__FILE__,__LINE__,
+	    mysql_error(&mysql));
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/********************************************
+rename the mail address for indicated mac address
+********************************************/
+int renameMailAddressInMngDb(char* macAddr, char* mailStr){
+
+  char queryStr[BUFFMAXLN];
+
+  /* prepare query string */
+  snprintf(queryStr, BUFFMAXLN, 
+	   "update macaddrs set mailAddress='%s' "
+	   " where (status='A' or status='P') and macAddress='%s'", 
+	   mailStr, macAddr);
+
+  /* send SQL query */
+  if (mysql_query(&mysql, queryStr)){
+    err_msg("ERR at %s#%d: mysql query: %s",__FILE__,__LINE__,
+	    mysql_error(&mysql));
+    return FALSE;
+  }
+  return TRUE;
+}
+
+/********************************************
+Register or Update MacAddress which is unlinked to user
+********************************************/
+int regOrUpNobodyMacAddr(char* macAddress){
+
+  int modified=FALSE;
+
+  /* macAddress in inactive status cannot be modified */
+  if(IsMacAddrStatusInactiveInMngDb(macAddress)) return FALSE;
+  
+  /* if mac is already registered, update it */
+  if(IsMacAddrFoundInMngDb(macAddress)){
+    if(RenewMacAddrInMngDb(macAddress)){
+      PutMacModifyLogToMngDb("?", "", macAddress, 'E'); /*(userId,extraId,.)*/
+      modified=TRUE;
+    }
+  }
+
+  /* if mac is not yet registered, register it */ 
+  else{
+    if(RegistMacAddrToMngDb(macAddress,"?","?","","")){ /*(dev,user,ext,mail) */
+	PutMacModifyLogToMngDb("?","", macAddress, 'R'); /* (userId,extraId,.) */
+      modified=TRUE;
+    }
+  }
+  return modified;
+}
+
+/******************************************
+ Is the macAddr  in INACTIVE('I') status
+******************************************/
+int isMacAddrStatusInactiveInMngDb(char* macAddr){
+
+  MYSQL_RES *res=NULL;
+  MYSQL_ROW row;
+  char queryStr[BUFFMAXLN];
+  int ret;
+
+  /* prepare query string */
+  snprintf(queryStr, BUFFMAXLN, 
+	   "select * from macaddrs "
+	   " where macAddress='%s' and status='I'",
+	   macAddr);
+  
+  /* send SQL query */
+  if (mysql_query(&mysql, queryStr)){
+    err_msg("ERR at %s#%d: mysql query: %s",__FILE__,__LINE__,
+	    mysql_error(&mysql));
+    return FALSE;
+  }
+  
+  /* store result */
+  res = mysql_store_result(&mysql);
+
+  /* output table name */
+  row = mysql_fetch_row(res);
+  
+  /* if found, return true */
+  if(row!=NULL) ret=TRUE;
+
+  /* if not found, return false */
+  else ret=FALSE;
+
+  mysql_free_result(res);
+  return ret;
+}
+
 /********************************************
  routines for debugging output
 ********************************************/
@@ -616,7 +797,7 @@ int InitMngDb(void){
   int ret;
   if(debug>1) err_msg("DEBUG:=>initMngDb()");
   ret=initMngDb();
-  if(debug>1) err_msg("DEBUG:(%d)<=closeMngDb()",ret);
+  if(debug>1) err_msg("DEBUG:(%d)<=initMngDb()",ret);
   return ret;
 }
 void CloseMngDb(void){
@@ -680,11 +861,11 @@ int GetNicVendorFromMngDb(char* macAddress, char* vendor, int bufferLength){
 }
 
 
-int CountMacModifyPerDayInMngDb(char* userId, char* extraId){
+int CountMacModifyPerDayInMngDb(char* userId, char* extraId, char* macAddress){
   int ret;
-  if(debug>1) err_msg("DEBUG:=>countMacModifyPerDayInMngDb(%s,%s)",
-		      userId,extraId);
-  ret=countMacModifyPerDayInMngDb(userId,extraId);
+  if(debug>1) err_msg("DEBUG:=>countMacModifyPerDayInMngDb(%s,%s,%s)",
+		      userId,extraId,macAddress);
+  ret=countMacModifyPerDayInMngDb(userId,extraId,macAddress);
   if(debug>1) err_msg("DEBUG:(%d)<=countMacModifyPerDayInMngDb( )",ret);
   return ret;
 }
@@ -721,13 +902,14 @@ int IsSessionFoundInMngDb(char* macAddr){
   return ret;
 }
 
-int GetNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, char* deviceName, char* openTime, char* gatewayName, int* weekday){
+int GetNextUsageLogFromMngDb(char* userId, char* extraId, char* macAddr, 
+ char* deviceName, char* openTime, char* closeTime, char* gatewayName){
   int ret;
   if(debug>1) err_msg("DEBUG:=>getNextUsageLogFromMngDb(%s,%s)",userId,extraId);
   ret=getNextUsageLogFromMngDb(userId,extraId,macAddr,deviceName,
-			      openTime,gatewayName,weekday);
-  if(debug>1) err_msg("DEBUG:(%d)<=getNextUsageLogFromMngDb(,,%s,%s,%s,%s,%d)",
-		      ret,macAddr,deviceName,openTime,gatewayName,*weekday);
+			      openTime,closeTime,gatewayName);
+  if(debug>1) err_msg("DEBUG:(%d)<=getNextUsageLogFromMngDb(,,%s,%s,%s,%s,%s)",
+	         ret,macAddr,deviceName,openTime,closeTime,gatewayName);
   return ret;
 }
 
@@ -739,3 +921,45 @@ int GetNextMailAddressFromMngDb(char* mailAddress, char* limitDate, char*device)
 		      ret,mailAddress,limitDate,device);
   return ret;
 }
+
+int GetTimeRangeToShowLog(char* beginTime, char* endTime, int* dateCount){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>getTimeRangeToShowLog( )");
+  ret=getTimeRangeToShowLog(beginTime, endTime, dateCount);
+  if(debug>1) err_msg("DEBUG:(%d)<=getTimeRangeToShowLog(%s,%s,%d)",
+		      ret,beginTime,endTime,*dateCount);
+  return ret;
+}
+
+int RenameDeviceNameInMngDb(char* macAddr, char* nameStr){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>renameDevideNameInMngDb(%s,%s)",macAddr,nameStr);
+  ret=renameDeviceNameInMngDb(macAddr, nameStr);
+  if(debug>1) err_msg("DEBUG:(%d)<=renameDeviceNameInMngDb()",ret);
+  return ret;
+}
+
+int RenameMailAddressInMngDb(char* macAddr, char* mailStr){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>renameMailAddressInMngDb(%s,%s)",macAddr,mailStr);
+  ret=renameMailAddressInMngDb(macAddr, mailStr);
+  if(debug>1) err_msg("DEBUG:(%d)<=renameMailAddressInMngDb()",ret);
+  return ret;
+}
+
+int RegOrUpNobodyMacAddr(char* macAddress){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>regOrUpNobodyMacAddr(%s)",macAddress);
+  ret=regOrUpNobodyMacAddr(macAddress);
+  if(debug>1) err_msg("DEBUG:(%d)<=regOrUpNobodyMacAddr()",ret);
+  return ret;
+}
+
+
+int IsMacAddrStatusInactiveInMngDb(char* macAddress){
+    int ret;
+  if(debug>1) err_msg("DEBUG:=>isMacAddrStatusInactiveInMngDb(%s)",macAddress);
+  ret=isMacAddrStatusInactiveInMngDb(macAddress);
+  if(debug>1) err_msg("DEBUG:(%d)<=isMacAddrStatusInactiveInMngDb()",ret);
+  return ret;
+}
