@@ -1,6 +1,5 @@
 /**************************************************
-opengate Mac addr auth program
-
+OpengateM - MAC address authentication system 
    module for local work database
 
 Copyright (C) 2011 Opengate Project Team
@@ -25,6 +24,22 @@ Email: watanaby@is.saga-u.ac.jp
 #include "opengatemmng.h"
 #include <sqlite3.h>
 
+static int sqliteBusyTimeout=100;  /* value used in sqite3_busy_timeout() */
+
+/*******************************************************************
+ read sqlite busy timeout value from conf and set to static variable
+*******************************************************************/
+int setupSqliteBusyTimeoutValue(void){
+
+  char *str;
+  
+  /* if set in conf, use the value. if not, use the above default. */
+  str=GetConfValue("SqliteBusyTimeout");
+  if(str!=NULL) sqliteBusyTimeout=atoi(str);
+
+  return sqliteBusyTimeout;
+}
+
 /********************************************
 initialize work db implemented with sqlite
 ********************************************/
@@ -56,12 +71,16 @@ int initWorkDb(void){
     "(macAddress TEXT PRIMARY KEY ON CONFLICT REPLACE, "
     "detectTime INTEGER, ttl INTEGER, isNat INTEGER)";
 
+  /* setup static variable value for SqLite3_busy_timeout from conf */
+  SetupSqliteBusyTimeoutValue();
+
   /* Open sqlite for opengateMmng */
   if(sqlite3_open(GetConfValue("SqliteDbMmng"),&db)!=SQLITE_OK){
     err_msg("ERR at %s#%d: sqlite3_open",__FILE__,__LINE__);
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* create table */
   if(sqlite3_exec(db, createCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
@@ -84,6 +103,7 @@ int initWorkDb(void){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* create table3 */
   if(sqlite3_exec(db, createCmd3, NULL, NULL, &pErrMsg)!=SQLITE_OK){
@@ -118,6 +138,7 @@ int createMacCheckTableInWorkDb(void){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* renew mac check table */
    if(sqlite3_exec(db, delCmd, NULL, NULL, &pErrMsg)!=SQLITE_OK){
@@ -171,6 +192,7 @@ int addIpv4ToMacCheckTable(void){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* get queued items */
   while(Dequeue(macAddr, ipAddr)){
@@ -227,6 +249,7 @@ int addIpv6ToMacCheckTable(void){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* get queued items */
   while(Dequeue(macAddr, ipAddr)){
@@ -284,6 +307,7 @@ int getNextRowInMacCheckTable(char* macAddress, char* ipv4, char* ipv6){
       sqlite3_close(db);
       terminateProg(0);
     }
+    sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
     /* compile to internal statement */
     if(sqlite3_prepare(db, selectCmd, BUFFMAXLN, &stmt, NULL)!=SQLITE_OK){
@@ -299,9 +323,9 @@ int getNextRowInMacCheckTable(char* macAddress, char* ipv4, char* ipv6){
 
   /* get a record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(macAddress, (char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
-    strncpy(ipv4, (char*)sqlite3_column_text(stmt, 1), ADDRMAXLN);
-    strncpy(ipv6, (char*)sqlite3_column_text(stmt, 2), ADDRMAXLN);
+    strlcpy(macAddress, (char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
+    strlcpy(ipv4, (char*)sqlite3_column_text(stmt, 1), ADDRMAXLN);
+    strlcpy(ipv6, (char*)sqlite3_column_text(stmt, 2), ADDRMAXLN);
     resultFlag=TRUE;
   }
   
@@ -339,6 +363,7 @@ int getIpFromMacCheckTable(char* macAddress, char* ipv4, char* ipv6){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, macAddress);
@@ -357,8 +382,8 @@ int getIpFromMacCheckTable(char* macAddress, char* ipv4, char* ipv6){
 
   /* get a record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(ipv4, (char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
-    strncpy(ipv6, (char*)sqlite3_column_text(stmt, 1), ADDRMAXLN);
+    strlcpy(ipv4, (char*)sqlite3_column_text(stmt, 0), ADDRMAXLN);
+    strlcpy(ipv6, (char*)sqlite3_column_text(stmt, 1), ADDRMAXLN);
     resultFlag=TRUE;
   }
   
@@ -396,6 +421,7 @@ int getPidFromMacCheckTable(char* macAddress, int* pid, int* ruleIpv4, int* rule
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, macAddress);
@@ -450,6 +476,7 @@ int savePidToMacCheckTable(char* macAddress, int pid, int ruleIpv4, int ruleIpv6
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute update command */
   updateCmd=sqlite3_mprintf(updateFormat, pid,
@@ -484,6 +511,7 @@ int getDetectTimeFromMacinfoTable(char* macAddress){
     sqlite3_close(db);
     return 0;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, macAddress);
@@ -531,6 +559,7 @@ int isCookieFoundInWorkDb(char* cookie, char* userId, char* extraId, int userTyp
     sqlite3_close(db);
     return FALSE;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, cookie, userType);
@@ -549,8 +578,8 @@ int isCookieFoundInWorkDb(char* cookie, char* userId, char* extraId, int userTyp
 
   /* get a record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(userId, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
-    strncpy(extraId, (char*)sqlite3_column_text(stmt, 1), USERMAXLN);
+    strlcpy(userId, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
+    strlcpy(extraId, (char*)sqlite3_column_text(stmt, 1), USERMAXLN);
     resultFlag=TRUE;
   }
   else resultFlag=FALSE;
@@ -582,6 +611,7 @@ int isSessionFoundInSessionTable(char* macAddress){
     sqlite3_close(db);
     return 0;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, macAddress);
@@ -633,6 +663,7 @@ int saveCookieToWorkDb(char* cookie, char* userId, char* extraId, int userType){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute delete command */
   delCmd=sqlite3_mprintf(delFormat, time(NULL)-(60*60*24));
@@ -674,6 +705,7 @@ int saveMacForCookieToWorkDb(char* cookie, char* macAddress){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute update command */
   updateCmd=sqlite3_mprintf(updateFormat, macAddress, cookie);
@@ -707,6 +739,7 @@ int saveMailDefalutForCookieToWorkDb(char* cookie, char* mailDefault){
     sqlite3_close(db);
     terminateProg(0);
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute update command */
   updateCmd=sqlite3_mprintf(updateFormat, mailDefault, cookie);
@@ -739,6 +772,7 @@ int getMailDefaultFromWorkDb(char* cookie, char* mailDefault){
     sqlite3_close(db);
     return FALSE;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, cookie);
@@ -757,7 +791,7 @@ int getMailDefaultFromWorkDb(char* cookie, char* mailDefault){
 
   /* get a record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(mailDefault, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
+    strlcpy(mailDefault, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
     resultFlag=TRUE;
   }
   else resultFlag=FALSE;
@@ -787,6 +821,7 @@ int loadMacForCookieFromWorkDb(char* cookie, char* macAddress){
     sqlite3_close(db);
     return FALSE;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, cookie);
@@ -805,7 +840,7 @@ int loadMacForCookieFromWorkDb(char* cookie, char* macAddress){
 
   /* get a record */
   if(sqlite3_step(stmt)==SQLITE_ROW){
-    strncpy(macAddress, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
+    strlcpy(macAddress, (char*)sqlite3_column_text(stmt, 0), USERMAXLN);
     resultFlag=TRUE;
   }
   else resultFlag=FALSE;
@@ -841,6 +876,7 @@ int isNatSuspectedInWorkDb(char* macAddr){
     sqlite3_close(db);
     return FALSE;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* prepare command string */
   selectCmd=sqlite3_mprintf(selectFormat, macAddr);
@@ -895,6 +931,7 @@ int isActiveSessionFoundInOpengateSessionTable(char* macAddress){
     sqlite3_close(db);
     return 0;
   }
+  sqlite3_busy_timeout(db, sqliteBusyTimeout);
 
   /* execute command */
   selectCmd=sqlite3_mprintf(selectFormat, macAddress, macAddress);
@@ -926,6 +963,14 @@ int isActiveSessionFoundInOpengateSessionTable(char* macAddress){
 /*********************************************************
  routines for debugging output
 *********************************************************/
+int SetupSqliteBusyTimeoutValue(void){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>setupSqliteBusyTimeoutValue()");
+  ret=setupSqliteBusyTimeoutValue();
+  if(debug>1) err_msg("DEBUG:(%d)<=setupSqliteBusyTimeoutValue()",ret);
+  return ret;
+}
+
 int InitWorkDb(void){
   int ret;
   if(debug>1) err_msg("DEBUG:=>initWorkDb( )");
